# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _

CONTINENTS = (
    ('AF', _('Africa')),
    ('NA', _('North America')),
    ('EU',  _('Europe')),
    ('AS', _('Asia')),
    ('OC',  _('Oceania')),
    ('SA', _('South America')),
    ('AN', _('Antarctica'))
)

AREAS = (
    ('a', _('Another')),
    ('i', _('Island')),
    ('ar', _('Arrondissement')),
    ('at', _('Atoll')),
    ('ai', _('Autonomous island')),
    ('ca', _('Canton')),
    ('cm', _('Commune')),
    ('co', _('County')),
    ('dp', _('Department')),
    ('de', _('Dependency')),
    ('dt', _('District')),
    ('dv', _('Division')),
    ('em', _('Emirate')),
    ('gv', _('Governorate')),
    ('ic', _('Island council')),
    ('ig', _('Island group')),
    ('ir', _('Island region')),
    ('kd', _('Kingdom')),
    ('mu', _('Municipality')),
    ('pa', _('Parish')),
    ('pf', _('Prefecture')),
    ('pr', _('Province')),
    ('rg', _('Region')),
    ('rp', _('Republic')),
    ('sh', _('Sheading')),
    ('st', _('State')),
    ('sd', _('Subdivision')),
    ('sj', _('Subject')),
    ('ty', _('Territory')),
)



class Country(models.Model):
    """
    International Organization for Standardization (ISO) 3166-1 Country list
    """
    iso2_code = models.CharField(_('ISO alpha-2'), max_length=2, unique=True)
    name = models.CharField(_('Official name (CAPS)'), max_length=128)
    printable_name = models.CharField(_('Country name'), max_length=128)
    iso3_code = models.CharField(_('ISO alpha-3'), max_length=3, unique=True)
    numcode = models.PositiveSmallIntegerField(_('ISO numeric'), null=True, blank=True)
    active = models.BooleanField(_('Country is active'), default=True)
    continent = models.CharField(_('Continent'), choices=CONTINENTS, max_length=2)
    admin_area = models.CharField(_('Administrative Area'), choices=AREAS, max_length=2, null=True, blank=True)

    class Meta:
        verbose_name = _('Country')
        verbose_name_plural = _('Countries')
        ordering = ('name',)

    def __unicode__(self):
        return self.printable_name


class AdminArea(models.Model):
    """
    Administrative Area level 1 for a country.  For the US, this would be the states
    """
    country = models.ForeignKey(Country)
    name = models.CharField(_('Admin Area name'), max_length=60, )
    abbrev = models.CharField(_('Postal Abbreviation'), max_length=3, null=True, blank=True)
    active = models.BooleanField(_('Area is active'), default=True)

    class Meta:
        verbose_name = _('Administrative Area')
        verbose_name_plural = _('Administrative Areas')
        ordering = ('name',)

    def __unicode__(self):
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2011 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved.
#
# Please see the "LICENSE" and "NOTICE" files included in the source 
# distribution for details on licensing info.

from configobj import Section
from notmm.utils.configparse import *

from test_support import unittest

class UserConfigTestCase(unittest.TestCase):
    
    def setUp(self):
        pass

    def test_loadconf(self):
        config = loadconf('development.ini')
        self.assertEqual(isinstance(config, dict), True)
        
        section = loadconf('development.ini', section='blogengine')
        self.assertEqual(isinstance(section, Section), True)

    def test_string_getter(self):
        app_conf = loadconf('development.ini') 
        s = string_getter(app_conf, 'blogengine', 'schevo.db.blogengine')
        self.assertEqual(isinstance(s, str), True, repr(s))

    def tearDown(self):
        pass




__author__="gary paulson"
__date__ ="$Jan 4, 2009 12:55:44 PM$"

from django import template
from satchmo_utils.templatetags import get_filter_args
from satchmo_utils.numbers import round_decimal, RoundedDecimalError

try:
    from django.utils.safestring import mark_safe
except ImportError:
    mark_safe = lambda s:s

import logging

log = logging.getLogger("satchmo_currency")

register = template.Library()

def normalize_decimal(value, args=""):
    """
    PARTIAL UNIT ROUNDING DECIMAL
    Converts a valid float, integer, or string to a decimal number with a specified
    number of decimal places, performs "partial unit rounding", and decimal normalization.

    Usage:
        val|normalize_decimal
        val|normalize_decimal:'places=2'
        val|normalize_decimal:'places=2:roundfactor=.5'
        val|normalize_decimal:'places=2:roundfactor=.5:normalize=False'

    val
        The value to be converted and optionally formated to decimal.
    places 
        The decimal place precision is defined by integer "places" and
        must be <= the precision defined in the decimal.Decimal context.  roundfactor represents
        the maximum number of decimal places to display if normalize is False.

    roundfactor
        (partial unit rounding factor) If roundfactor is between 0 and 1, roundfactor rounds up
        (positive roundfactor value) or down (negative roundfactor value) in factional "roundfactor" increments.
    
    normalize
        If normalize is True (any value other than False), then rightmost zeros are truncated.

    General Filter/Template Usage.
        normalize_decimal is generally used without parameters in the template.
        Defaults are: places=2, roundfactor=None, normalize=True
        If normalize_decimal is not used as a template filter, the value (quantity)
        will display the full decimal value in the template field.
    """
    
    if value == '' or value is None:
        return value
    args, kwargs = get_filter_args(args,
        keywords=('places','roundfactor', 'normalize'),
        intargs=('places',),
        boolargs=('normalize',), stripquotes=True)

    if not 'places' in kwargs:
        kwargs['places'] = 2

    try:
        return mark_safe(str(round_decimal(val=value, **kwargs)))

    except RoundedDecimalError, e:
        log.error("normalize_decimal error val=%s, id-%s, msg=%s", (e.val, e.id, e.msg))
        return value

register.filter('normalize_decimal', normalize_decimal)
normalize_decimal.is_safe = True



#Global settings for the helloworld app, to be overrided in local_settings.py.
from django.conf.global_settings import *
from pkg_resources import resource_filename
MEDIA_ROOT = resource_filename('helloworld', 'static')
ROOT_URLCONF='helloworld.config.urls'
#ENABLE_BEAKER=False
MEDIA_URL="http://localhost/media/img/"
SECRET_KEY='12345va1110ht'
#ENABLE_TIDYLIB=False
DEBUG=True

#I18N stuff (experimental)
USE_I18N=False
LANGUAGE_CODE = 'fr-CA'
DEFAULT_CHARSET = 'utf-8'
DEFAULT_CONTENT_TYPE = 'text/html; charset=%s' % DEFAULT_CHARSET

# Custom error handlers mapping
CUSTOM_ERROR_HANDLERS = (
    ('handle302', 'helloworld.handlers.handle302'),
    ('handle404', 'helloworld.handlers.handle404'),
    ('handle500', 'helloworld.handlers.handle500'),
)

# For backward-compatibility with Django.
# Import templates from the package name.
TEMPLATE_DIRS = (
    (resource_filename('helloworld', 'templates')),
)
TEMPLATE_LOADER = 'notmm.utils.template.backends.CachedTemplateLoader'

# Logging options
LOGGING_FORMAT = '[%(levelname)-5s] - [%(asctime)-15s] - [%(name)-5s] - %(message)s' 
# Where to send application errors 
LOGGING_ERROR_LOG = '/var/log/django.log'

TEMPLATE_CONTEXT_PROCESSORS = (
    'helloworld.config.context_processors.request',
    )
OAUTH2_CLIENT_ID = '51290408562-abs4t4ecqo834diltkk3d42cef6fi2sd.apps.googleusercontent.com'
OAUTH2_ACCESS_TOKEN = 'd5RM8aoiHBET3Fu5kUBqwDGg'
OAUTH2_REDIRECT_URL = 'http://localhost:8000/oauth2callback'




# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk
import pango
from kiwi.utils import gsignal, type_register

from gazpacho.command import Command
from gazpacho.commandmanager import command_manager
from gazpacho.i18n import _

(MODE_NOTHING,
 MODE_ADD,
 MODE_REMOVE,
 MODE_CHANGE) = list(range(4))

(COLUMN_SIGNAL,
 COLUMN_HANDLER,
 COLUMN_AFTER,
 COLUMN_AFTER_VISIBLE,
 COLUMN_HANDLER_EDITABLE,
 COLUMN_SLOT,
 COLUMN_OBJECT,
 COLUMN_BOLD) = list(range(8))

class SignalInfo(object):
    """
    This class encapsulates information about a signal handler.

    In addition to the actual handler information this class also
    stores the handler name in a normalized form where all underscores
    have been replaced with dashes. For example, if the name is
    'my_signal' the normalized name will be 'my-signal'. When
    comparing SignalInfo instances this will be based on the
    normalized name and not the name.

    @cvar name:
    @cvar normalized_name:
    @cvar handler:
    @cvar after:
    @cvar object: object
    """
    def __init__(self, name, handler, after=False, object=''):
        if not isinstance(name, str):
            raise TypeError("name must be a string not %r" % name)
        self.name = name
        self.normalized_name = name.replace('_', '-')
        if handler is not None and not isinstance(handler, str):
            raise TypeError(
                "handler must be a string or None, not %r" % handler)
        self.handler = handler
        if not isinstance(after, bool):
            raise TypeError("after must be a bool, not %r" % after)
        self.after = after
        if object is not None and not isinstance(object, str):
            raise TypeError("object must be a string or None, not %r" % object)
        self.object = object

    def copy(self):
        return self.__class__(name=self.name, handler=self.handler,
                              after=self.after, object=self.object)

    def __repr__(self):
        return '<SignalInfo name=%s, handler=%s %s %s>' % (self.name,
                                                           self.handler,
                                                           self.after,
                                                           self.object)

    def __eq__(self, other):
        """
        Compare if two SignalInfo instance are considered equal. Note
        that this is not based on the handler name but the normalized
        name.
        """
        if not isinstance(other, self.__class__):
            return False
        if ((self.normalized_name, self.handler, self.after, self.object) !=
            (other.normalized_name, other.handler, other.after, other.object)):
            return False
        return True

SLOT_MESSAGE = _("<Type the signal's handler here>")

class SignalEditor(gtk.VBox):
    """ The GladeSignalEditor is used to house the signal editor interface and
    associated functionality """

    gsignal('signal-activated', object, str)

    def __init__(self, editor, app):
        gtk.VBox.__init__(self)

        self._editor = editor
        self._app = app
        self._model = None
        self._gadget = None
        self._adaptor = None
        self._model = gtk.TreeStore(str, str, bool, bool,
                                    bool, bool, str, bool)
        self._treeview = self._construct_signal_list(self._model)

        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.add(self._treeview)
        self.pack_start(scroll)
        self.show_all()

    def load_gadget(self, gadget):
        """
        Loads all the signals from the gadget and displays them in the view

        @param gadget: a L{gazpacho.gadget.Widget} instance
        """
        self._gadget = gadget
        self._adaptor = gadget and gadget.adaptor or None

        self._internal_load(gadget)

    def _construct_signal_list(self, model):
        view = gtk.TreeView(model)
        view.connect('row-activated', self._treeview_row_activated_cb)
        view.connect('key-press-event', self._treeview_key_press_event_cb)
        view.connect('button-press-event', self._treeview_button_press_event_cb)

        # signal column
        renderer = gtk.CellRendererText()
        renderer.set_property('weight', pango.WEIGHT_BOLD)
        column = gtk.TreeViewColumn(_('Signal'), renderer, text=COLUMN_SIGNAL,
                                    weight_set=COLUMN_BOLD)
        view.append_column(column)

        # handler column
        renderer = gtk.CellRendererText()
        renderer.set_property('style', pango.STYLE_ITALIC)
        renderer.set_property('foreground', 'Gray')
        renderer.connect('edited', self._handler_cell_edited_cb)
        column = gtk.TreeViewColumn(_('Handler'), renderer,
                                    text=COLUMN_HANDLER,
                                    style_set=COLUMN_SLOT,
                                    foreground_set=COLUMN_SLOT,
                                    editable=COLUMN_HANDLER_EDITABLE)
        view.append_column(column)

        # after column
        renderer = gtk.CellRendererToggle()
        renderer.connect('toggled', self._after_cell_toggled_cb)
        column = gtk.TreeViewColumn(_('After'), renderer,
                                    active=COLUMN_AFTER,
                                    visible=COLUMN_AFTER_VISIBLE)
        view.append_column(column)

        # object column
        renderer = gtk.CellRendererText()
        renderer.connect('edited', self._object_cell_edited_cb)
        column = gtk.TreeViewColumn(_('Object'), renderer,
                                    editable=COLUMN_HANDLER_EDITABLE,
                                    text=COLUMN_OBJECT,
                                    visible=COLUMN_AFTER_VISIBLE)
        view.append_column(column)

        return view

    def _internal_load(self, gadget):
        model = self._model
        model.clear()

        if not gadget:
            return

        last_type = ""
        parent_class = None
        for signal_name, signal_type in self._adaptor.list_signals():
            if signal_type != last_type:
                parent_class = model.append(None)
                model.set(parent_class,
                          COLUMN_SIGNAL, signal_type,
                          COLUMN_AFTER_VISIBLE, False,
                          COLUMN_HANDLER_EDITABLE, False,
                          COLUMN_SLOT, False,
                          COLUMN_BOLD, False)
                last_type = signal_type

            parent_signal = model.append(parent_class)
            signals = gadget.list_signal_handlers(signal_name)

            if not signals:
                model.set(parent_signal,
                          COLUMN_SIGNAL, signal_name,
                          COLUMN_HANDLER, SLOT_MESSAGE,
                          COLUMN_AFTER, False,
                          COLUMN_HANDLER_EDITABLE, True,
                          COLUMN_AFTER_VISIBLE, False,
                          COLUMN_SLOT, True)
            else:
                gadget_signal = signals[0]
                model.set(parent_signal,
                          COLUMN_BOLD, True,
                          COLUMN_SIGNAL, signal_name,
                          COLUMN_HANDLER, gadget_signal.handler,
                          COLUMN_AFTER, gadget_signal.after,
                          COLUMN_OBJECT, gadget_signal.object,
                          COLUMN_AFTER_VISIBLE, True,
                          COLUMN_HANDLER_EDITABLE, True,
                          COLUMN_SLOT, False)
                self._treeview.expand_row(model[parent_class].path, False)

                for widget_signal in signals[1:]:
                    model.set(model.append(parent_signal),
                              COLUMN_HANDLER, widget_signal.handler,
                              COLUMN_AFTER, widget_signal.after,
                              COLUMN_OBJECT, widget_signal.object,
                              COLUMN_AFTER_VISIBLE, True,
                              COLUMN_HANDLER_EDITABLE, True,
                              COLUMN_SLOT, False)

                # add the <Type...> slot
                model.set(model.append(parent_signal),
                          COLUMN_HANDLER, SLOT_MESSAGE,
                          COLUMN_AFTER, False,
                          COLUMN_OBJECT, '',
                          COLUMN_AFTER_VISIBLE, False,
                          COLUMN_HANDLER_EDITABLE, True,
                          COLUMN_SLOT, True)

        self._treeview.expand_row((0,), False)

    def _is_valid_identifier(self, text):
        if text is None:
            return False

        return True

    def _append_slot(self, iter_signal):
        model = self._model
        model.set(model.append(iter_signal),
                  COLUMN_HANDLER, SLOT_MESSAGE,
                  COLUMN_AFTER, False,
                  COLUMN_OBJECT, '',
                  COLUMN_AFTER_VISIBLE, False,
                  COLUMN_HANDLER_EDITABLE, True,
                  COLUMN_SLOT, True)

        # mark the signal and class name as bold
        row = model[iter_signal]
        row[COLUMN_BOLD] = True
        row.parent[COLUMN_BOLD] = True

    def _remove_row(self, model_iter):
        "Remove a row from the signal tree"
        model = self._model
        # if this isn't the first row we can just remove it
        signal_name = model[model_iter][COLUMN_SIGNAL]
        if signal_name is None:
            model.remove(model_iter)
            return

        # We cannot delete the first row so we have to copy the next
        # row and delete that instead
        next_iter = model.iter_nth_child(model_iter, 0)
        if not next_iter:
            return

        (handler, after, visible,
         editable, slot, obj) = tuple(model[next_iter])[1:7]
        row = model[model_iter]
        row[COLUMN_HANDLER] = handler
        row[COLUMN_AFTER] = after
        row[COLUMN_AFTER_VISIBLE] = visible
        row[COLUMN_HANDLER_EDITABLE] = editable
        row[COLUMN_OBJECT] = obj
        row[COLUMN_SLOT] = slot

        next_iter = model.remove(next_iter)

        # if last signal handler was removed, the parents has to be updated
        # to reflect this
        if not next_iter:
            row = model[model_iter]
            row[COLUMN_BOLD] = False

            # We need to go through all children of the signal class
            # and see if any of them are bold and update the class node
            # to reflect this
            bold = False
            for child in row.parent.iterchildren():
                bold = child[COLUMN_BOLD]
                if bold:
                    break

            row.parent[COLUMN_BOLD] = bold

    def _is_row_deletable(self, model_iter):
        """Check whether a row can be deleted or not."""
        row = self._model[model_iter]
        return row[COLUMN_HANDLER_EDITABLE] and not row[COLUMN_SLOT]

    def _remove_signal_handler(self, model_iter):
        """Remove signal handler both from the treeview and the widget."""
        row = self._model[model_iter]
        signal_name = row[COLUMN_SIGNAL]
        if signal_name is None:
            signal_name = row.parent[COLUMN_SIGNAL]

        signal = SignalInfo(name=signal_name,
                            handler=row[COLUMN_HANDLER],
                            after=row[COLUMN_AFTER],
                            object=row[COLUMN_OBJECT])
        cmd = CommandAddRemoveSignal(self._gadget, signal, False)
        command_manager.execute(cmd, self._gadget.project)
        self._remove_row(model_iter)

    def _create_popup(self, item_iter):
        def _popup_delete_activate_cb(item, item_iter):
            self._remove_signal_handler(item_iter)

        menu = gtk.Menu()
        menu_item = gtk.ImageMenuItem(gtk.STOCK_DELETE, None)
        menu_item.connect('activate', _popup_delete_activate_cb, item_iter)
        menu_item.show()
        menu.append(menu_item)
        return menu

    def _is_void_signal_handler(self, signal_handler):
        return (signal_handler is None or
                signal_handler.strip() == '' or
                signal_handler == SLOT_MESSAGE)

    def _handler_cell_edited_cb(self, cell, path_str, new_handler):
        if (self._model[path_str][COLUMN_SLOT] and
            self._is_void_signal_handler(new_handler)):
            return
        self._change(path_str, handler=new_handler)

    def _object_cell_edited_cb(self, cell, path_str, obj):
        if self._model[path_str][COLUMN_SLOT] and not obj:
            return
        self._change(path_str, obj=obj)

    def _change(self, path, handler=None, obj=None):
        if handler is None and obj is None:
            raise TypeError

        model = self._model
        model_iter = model[path].iter
        (signal_name, old_handler,
         after, slot, old_object) = model.get(model_iter,
                                              COLUMN_SIGNAL, COLUMN_HANDLER,
                                              COLUMN_AFTER, COLUMN_SLOT,
                                              COLUMN_OBJECT)
        if signal_name is None:
            row = model[model_iter]
            signal_name = row.parent[COLUMN_SIGNAL]
            iter_signal = row.parent.iter
        else:
            iter_signal = model_iter

        signal = SignalInfo(name=signal_name, handler=handler,
                            after=after, object=obj)

        if handler:
            is_void = (handler is None or
                       handler.strip() == '' or
                       handler == SLOT_MESSAGE)
        elif obj:
            is_void = (obj is None or obj.strip() == '')
        else:
            return

        if slot and not is_void:
            mode = MODE_ADD
        if not slot and is_void:
            mode = MODE_REMOVE
        if not slot and not is_void:
            mode = MODE_CHANGE

        # we are adding a new handler
        if mode == MODE_ADD:
            signal.after = False
            cmd = CommandAddRemoveSignal(self._gadget, signal, True)
            command_manager.execute(cmd, self._gadget.project)
            row = model[model_iter]
            row[COLUMN_SLOT] = False
            row[COLUMN_HANDLER] = handler
            row[COLUMN_AFTER_VISIBLE] = True
            row[COLUMN_OBJECT] = obj
            # append a <Type...> slot
            self._append_slot(iter_signal)

        # we are removing a signal handler
        if mode == MODE_REMOVE:
            signal.handler = old_handler
            signal.object = old_object
            cmd = CommandAddRemoveSignal(self._gadget, signal, False)
            command_manager.execute(cmd, self._gadget.project)
            self._remove_row(model_iter)
            return

        # we are changing a signal handler
        if mode == MODE_CHANGE:
            old_signal = signal.copy()
            old_signal.handler = old_handler
            old_signal.object = old_object
            if object:
                signal.handler = handler
            if handler:
                signal.object = obj
            row = model[model_iter]
            row[COLUMN_SLOT] = False
            row[COLUMN_HANDLER] = signal.handler
            row[COLUMN_AFTER_VISIBLE] = True
            row[COLUMN_OBJECT] = signal.object
            cmd = CommandChangeSignal(self._gadget, old_signal, signal)
            command_manager.execute(cmd, self._gadget.project)

    def _after_cell_toggled_cb(self, cell, path_str, data=None):
        model = self._model
        row = model[path_str]
        signal_name = row[COLUMN_SIGNAL]
        if signal_name is None:
            signal_name = row.parent[COLUMN_SIGNAL]
        handler = row[COLUMN_HANDLER]
        after = row[COLUMN_AFTER]
        obj = row[COLUMN_OBJECT]

        old_signal = SignalInfo(name=signal_name, object=obj,
                                handler=handler, after=after)
        new_signal = SignalInfo(name=signal_name, object=obj,
                                handler=handler, after=not after)

        cmd = CommandChangeSignal(self._gadget, old_signal, new_signal)
        command_manager.execute(cmd, self._gadget.project)

        row[COLUMN_AFTER] = not after

    def _treeview_row_activated_cb(self, view, path, column, data=None):
        view.set_cursor(path, column, True)
        view.grab_focus()
        self.emit('signal-activated', self._gadget,
                  self._model[path][COLUMN_SIGNAL])

    def _treeview_key_press_event_cb(self, view, event):
        if not event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete):
            return

        model_iter = view.get_selection().get_selected()[1]
        if self._is_row_deletable(model_iter):
            self._remove_signal_handler(model_iter)

    def _treeview_button_press_event_cb(self, view, event):
        if event.button != 3:
            return False

        result = view.get_path_at_pos(int(event.x), int(event.y))
        if not result:
            return False
        path = result[0]
        item_iter = self._model[path].iter

        if self._is_row_deletable(item_iter):
            view.set_cursor(path)
            view.grab_focus()
            popup = self._create_popup(item_iter)
            popup.popup(None, None, None, event.button, event.time)
            return True

        return False

type_register(SignalEditor)

# signal command
class CommandAddRemoveSignal(Command):
    def __init__(self, gadget, signal, add):
        if add:
            description = _('Add signal handler %s') % signal.handler
        else:
            description = _('Remove signal handler %s') % signal.handler

        Command.__init__(self, description)
        self._add = add
        self._signal = signal
        self._gadget = gadget

    def execute(self):
        if self._add:
            self._gadget.add_signal_handler(self._signal)
        else:
            self._gadget.remove_signal_handler(self._signal)

        self._add = not self._add

class CommandChangeSignal(Command):
    def __init__(self, gadget, old_signal_handler, new_signal_handler):
        description = _('Change signal handler for signal "%s"') % \
                      old_signal_handler.name
        Command.__init__(self, description)
        self._gadget = gadget
        self._old_handler = old_signal_handler
        self._new_handler = new_signal_handler

    def execute(self):
        self._gadget.change_signal_handler(self._old_handler,
                                                    self._new_handler)
        self._old_handler, self._new_handler = (self._new_handler,
                                                self._old_handler)



"""
SSL Middleware
Stephen Zabel

This middleware answers the problem of redirecting to (and from) a SSL secured path
by stating what paths should be secured in urls.py file. To secure a path, add the
additional view_kwarg 'SSL':True to the view_kwargs.

For example

urlpatterns = patterns('some_site.some_app.views',
    (r'^test/secure/$','test_secure',{'SSL':True}),
     )

All paths where 'SSL':False or where the kwarg of 'SSL' is not specified are routed
to an unsecure path.

For example

urlpatterns = patterns('some_site.some_app.views',
    (r'^test/unsecure1/$','test_unsecure',{'SSL':False}),
    (r'^test/unsecure2/$','test_unsecure'),
     )

Gotcha's : Redirects should only occur during GETs; this is due to the fact that
POST data will get lost in the redirect.

A major benefit of this approach is that it allows you to secure django.contrib views
and generic views without having to modify the base code or wrapping the view.

This method is also better than the two alternative approaches of adding to the
settings file or using a decorator.

It is better than the tactic of creating a list of paths to secure in the settings
file, because you DRY. You are also not forced to consider all paths in a single
location. Instead you can address the security of a path in the urls file that it
is resolved in.

It is better than the tactic of using a @secure or @unsecure decorator, because
it prevents decorator build up on your view methods. Having a bunch of decorators
makes views cumbersome to read and looks pretty redundant. Also because the all
views pass through the middleware you can specify the only secure paths and the
remaining paths can be assumed to be unsecure and handled by the middleware.

This package is inspired by Antonio Cavedoni's SSL Middleware

Satchmo notes:
This package has also merged the main concepts of Antonio Cavedoni's SSL Middleware, 
to allow for better integration with other sites, and to easily allow admin pages to
be secured.

Lastly, we've added an optional "SSL_PORT" to be specified in the settings, for
unusual server configurations.  If specified, the port will be sent with the
SSL redirect.
"""

__license__ = "Python"
__copyright__ = "Copyright (C) 2007, Stephen Zabel"
__author__ = "Stephen Zabel"

from django.conf import settings
from django.http import HttpResponseRedirect, get_host
from satchmo_utils import request_is_secure

HTTPS_PATHS = getattr(settings, "HTTPS_PATHS", [])
SSL = 'SSL'
SSLPORT=getattr(settings, 'SSL_PORT', None)

class SSLRedirect:
    def process_view(self, request, view_func, view_args, view_kwargs):
        if SSL in view_kwargs:
            secure = view_kwargs[SSL]
            del view_kwargs[SSL]
        else:
            secure = False
            
        if not secure:
            for path in HTTPS_PATHS:
                if request.path.startswith("/%s" % path):
                    secure = True
                    break

        if not secure == request_is_secure(request):
            return self._redirect(request, secure)

    def _redirect(self, request, secure):
        if settings.DEBUG and request.method == 'POST':
            raise RuntimeError(
"""Django can't perform a SSL redirect while maintaining POST data.
Please structure your views so that redirects only occur during GETs.""")

        protocol = secure and "https" or "http"
        host = "%s://%s" % (protocol, get_host(request))
        # In certain proxying situations, we need to strip out the 443 port
        # in order to prevent inifinite redirects
        if not secure:
            host = host.replace(':443','')
        if secure and SSLPORT:
            host = "%s:%s" % (host, SSLPORT)
            
        newurl = "%s%s" % (host, request.get_full_path())

        return HttpResponseRedirect(newurl)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) 2007-2012 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved.
# <LICENSE=ISC>
"""This module provide a convenient access to YAML files using the ``pyyaml``
extension as a Python dictionary.

Usage:
Require the presence of a ``CUSTOM_DATADIR`` setting to the value
of the directory holding the YAML files.`"""

import os, yaml, random
import posixpath

try:
    from io import open
except ImportError:
    pass

__all__ = ['YAMLDct', 'YAMLDctError', 'SortedYAMLDct', 'RandomYAMLDct']

class YAMLDctError(Exception):
    pass

class YAMLDct(object):
    """Returns a dict instance populated with the values 
    found in the YAML file"""

    def __init__(self, *args, **kwargs):
        pass

    def __new__(cls, path):
        return cls.openyamlfile(path)

    @classmethod
    def openyamlfile(cls, path):
        try:
            p = posixpath.realpath(path)
            with open(p, 'r') as f:
                aLst = yaml.load(f)
            f.close()    
        except (IOError, OSError) as error:
            raise YAMLDctError(error)
        return aLst

class SortedYAMLDct(YAMLDct):
    
    @classmethod
    def openyamlfile(cls, path):
        p = super(SortedYAMLDct, cls).openyamlfile(path)
        return sorted((item for item in p.iteritems()))

class RandomYAMLDct(YAMLDct):
    bits = 1
    
    @classmethod
    def openyamlfile(cls, path):
        p = super(RandomYAMLDct, cls).openyamlfile(path)
        random.seed(cls.bits)
        random.shuffle(p)
        return p




####################################################################
# Second step in the order process: Capture the billing method and shipping type
#####################################################################

from django import http
from django.conf import settings
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from livesettings import config_value
from satchmo_store.contact.models import Contact
from payment.config import gateway_live
from payment.forms import CreditPayShipForm, SimplePayShipForm
from product.utils import find_best_auto_discount
from satchmo_store.shop.models import Cart
from satchmo_store.shop.models import Order, OrderPayment
from satchmo_utils.dynamic import lookup_url, lookup_template

import logging

log = logging.getLogger('payship')
selection = _("Please Select")

def pay_ship_info_verify(request, payment_module):
    """Verify customer and cart.
    Returns:
    True, contact, cart on success
    False, destination of failure
    """
    # Verify that the customer exists.
    try:
        contact = Contact.objects.from_request(request, create=False)
    except Contact.DoesNotExist:
        log.debug('No contact, returning to step 1 of checkout')
        url = lookup_url(payment_module, 'satchmo_checkout-step1')
        return (False, http.HttpResponseRedirect(url))

    # Verify that we still have items in the cart.
    tempCart = Cart.objects.from_request(request)
    if tempCart.numItems == 0:
        template = lookup_template(payment_module, 'shop/checkout/empty_cart.html')
        return (False, render_to_response(template,
                                          context_instance=RequestContext(request)))

    return (True, contact, tempCart)

def credit_pay_ship_process_form(request, contact, working_cart, payment_module, allow_skip=True, *args, **kwargs):
    """Handle the form information.
    Returns:
        (True, destination) on success
        (False, form) on failure
    """
    
    def _get_form(request, payment_module, *args, **kwargs):
        processor = payment_module.MODULE.load_module('processor')
        log.debug('processor=%s', processor)
        if hasattr(processor, 'FORM'):
            log.debug('getting form from module')
            formclass = processor.FORM
        else:
            log.debug('using default form')
            formclass = CreditPayShipForm
        
        form = formclass(request, payment_module, *args, **kwargs)
        return form

    if request.method == "POST":
        new_data = request.POST.copy()
        
        form = _get_form(request, payment_module, new_data, *args, **kwargs)
        if form.is_valid():
            data = form.cleaned_data
            form.save(request, working_cart, contact, payment_module, data=data)
            url = lookup_url(payment_module, 'satchmo_checkout-step3')
            return (True, http.HttpResponseRedirect(url))
        else:
            log.debug('Form errors: %s', form.errors)
    else:
        order_data = {}
        try:
            order = Order.objects.from_request(request)
            if order.shipping_model:
                order_data['shipping'] = order.shipping_model
            if order.credit_card:
                cc = order.credit_card
                val = cc.decryptedCC
                if val:
                    order_data['credit_number'] = val
                val = cc.ccv
                if val:
                    order_data['ccv'] = val
                val = cc.expire_month
                if val:
                    order_data['month_expires'] = val
                val = cc.expire_year
                if val:
                    order_data['year_expires'] = val
                val = cc.credit_type
                if val:
                    order_data['credit_type'] = val
            
            kwargs['initial'] = order_data
            ordershippable = order.is_shippable
        except Order.DoesNotExist:
            pass
        
        form = _get_form(request, payment_module, *args, **kwargs)
        if not form.is_needed():
            log.debug('Skipping pay ship because form is not needed, nothing to pay')
            form.save(request, working_cart, contact, None, 
                data={'shipping' : form.shipping_dict.keys()[0]})

            url = lookup_url(payment_module, 'satchmo_checkout-step3')
            return (True, http.HttpResponseRedirect(url))
        
    return (False, form)

def simple_pay_ship_process_form(request, contact, working_cart, payment_module, allow_skip=True):
    if request.method == "POST":
        new_data = request.POST.copy()
        form = SimplePayShipForm(request, payment_module, new_data)
        if form.is_valid():
            form.save(request, working_cart, contact, payment_module)
            url = lookup_url(payment_module, 'satchmo_checkout-step3')
            return (True, http.HttpResponseRedirect(url))
        else:
            return (False, form)
    else:
        order_data = None
        try:
            order = Order.objects.from_request(request)
            if order.shipping_model:
                order_data = {}
                order_data['shipping'] = order.shipping_model
            ordershippable = order.is_shippable
        except Order.DoesNotExist:
            pass

        form = SimplePayShipForm(request, payment_module, order_data)
        if allow_skip:
            skipping = False
            skipstep = form.shipping_hidden or not ordershippable or (len(form.shipping_dict) == 1)
            if skipstep:               
                log.debug('Skipping pay ship, nothing to select for shipping')
                # no shipping choice = skip this step
                form.save(request, working_cart, contact, payment_module, 
                    data={'shipping' : form.fields['shipping'].initial})
                skipping = True
            elif not form.is_needed():
                log.debug('Skipping pay ship because form is not needed, nothing to pay')
                form.save(request, working_cart, contact, None, 
                    data={'shipping' : form.shipping_dict.keys()[0]})
                skipping = True
            
            if skipping:
                url = lookup_url(payment_module, 'satchmo_checkout-step3')
                return (True, http.HttpResponseRedirect(url))
                
        return (False, form)

def pay_ship_render_form(request, form, template, payment_module, cart):
    template = lookup_template(payment_module, template)
            
    ctx = RequestContext(request, {
        'form': form,
        'PAYMENT_LIVE': gateway_live(payment_module),
        })
    return render_to_response(template, context_instance=ctx)

def base_pay_ship_info(request, payment_module, form_handler, template):
    results = pay_ship_info_verify(request, payment_module)
    if not results[0]:
        return results[1]

    contact = results[1]
    working_cart = results[2]

    results = form_handler(request, contact, working_cart, payment_module)
    if results[0]:
        return results[1]

    form = results[1]
    return pay_ship_render_form(request, form, template, payment_module, working_cart)

def credit_pay_ship_info(request, payment_module, template='shop/checkout/pay_ship.html'):
    """A pay_ship view which uses a credit card."""
    return base_pay_ship_info(request, payment_module, credit_pay_ship_process_form, template)

def simple_pay_ship_info(request, payment_module, template):
    """A pay_ship view which doesn't require a credit card."""
    return base_pay_ship_info(request, payment_module, simple_pay_ship_process_form, template)



#!/usr/bin/env python
#from schevo.database2 import Database
from schevo.database import format_dbclass, equivalent
from test_support import unittest
from notmm.dbapi.orm import ClientStorageProxy

class ClientStorageTestCase(unittest.TestCase):

    connected = False

    def setUp(self):
        self.dbname = '127.0.0.1:4343'
        ClientStorageProxy.cache.clear()

    def tearDown(self):
        pass

    def test_initdb(self):
        db = ClientStorageProxy(self.dbname)
        #db.initdb(db.conn)
        #db2 = ZODBFileStorageProxy(self.dbname)
        #self.assertEqual(equivalent(db, db2), True)
        self.assertEqual(db != None, True)

        print db.extents()



from django.conf.urls.defaults import *
from livesettings import config_get_group

config = config_get_group('PAYMENT_GIFTCERTIFICATE')

urlpatterns = patterns('',
     (r'^$', 'payment.modules.giftcertificate.views.pay_ship_info', {'SSL':config.SSL.value}, 'GIFTCERTIFICATE_satchmo_checkout-step2'),
     (r'^confirm/$', 'payment.modules.giftcertificate.views.confirm_info', {'SSL':config.SSL.value}, 'GIFTCERTIFICATE_satchmo_checkout-step3'),
     (r'^success/$', 'payment.views.checkout.success', {'SSL':config.SSL.value}, 'GIFTCERTIFICATE_satchmo_checkout-success'),
     (r'^balance/$', 'payment.modules.giftcertificate.views.check_balance', {}, 'satchmo_giftcertificate_balance'),
)



from django.contrib import admin
from shipping.modules.tieredweight.models import Carrier, WeightTier, Zone, ZoneTranslation
from shipping.modules.tieredweight.forms import CarrierAdminForm, ZoneAdminForm


class WeightTierInline(admin.TabularInline):
    model = WeightTier


class ZoneTranslationInline(admin.TabularInline):
    model = ZoneTranslation


class CarrierOptions(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('name', 'default_zone')
        }),
        ('Advanced options', {
            'classes': ('collapse',),
            'fields': ('active', 'ordering')
        }),
    )
    list_display = ['name', 'active', 'default_zone']
    form = CarrierAdminForm


class ZoneOptions(admin.ModelAdmin):
    list_filter = ['carrier',]
    list_display = ['carrier', 'name']
    filter_horizontal = ('countries',)
    inlines = [WeightTierInline, ZoneTranslationInline]
    form = ZoneAdminForm


admin.site.register(Carrier, CarrierOptions)
admin.site.register(Zone, ZoneOptions)


#
#   SERMEPA / ServiRed payments module for Satchmo
#
#   Author: Michal Salaban <michal (at) salaban.info>
#   with a great help of Fluendo S.A., Barcelona
#
#   Based on "Guia de comercios TPV Virtual SIS" ver. 5.18, 15/11/2008, SERMEPA
#   For more information about integration look at http://www.sermepa.es/
#
#   TODO: SERMEPA interface provides possibility of recurring payments, which
#   could be probably used for SubscriptionProducts. This module doesn't support it.
#
from django.conf.urls.defaults import *
from livesettings import config_get_group

config = config_get_group('PAYMENT_SERMEPA')

urlpatterns = patterns('',
    (r'^$', 'payment.modules.sermepa.views.pay_ship_info', {'SSL': config.SSL.value}, 'SERMEPA_satchmo_checkout-step2'),
    (r'^confirm/$', 'payment.modules.sermepa.views.confirm_info', {'SSL': config.SSL.value}, 'SERMEPA_satchmo_checkout-step3'),
    (r'^success/$', 'payment.views.checkout.success', {'SSL': config.SSL.value}, 'SERMEPA_satchmo_checkout-success'),
    (r'^failure/$', 'payment.views.checkout.failure', {'SSL': config.SSL.value}, 'SERMEPA_satchmo_checkout-failure'),
    (
        r'^notify/$',
        'payment.modules.sermepa.views.notify_callback',
        {'SSL': config.SSL.value},
        'SERMEPA_satchmo_checkout-notify_callback'
        ),
    (r'^confirmorder/$', 'payment.views.confirm.confirm_free_order', 
       {'SSL' : config.SSL.value, 'key' : 'SERMEPA'}, 'SERMEPA_satchmo_checkout_free-confirm')
)



"""
Each shipping option uses the data in an Order object to calculate the shipping cost and return the value
"""
from decimal import Decimal
from django.utils.translation import ugettext, ugettext_lazy
from livesettings import config_value
_ = ugettext_lazy
from shipping.modules.base import BaseShipper

class Shipper(BaseShipper):
    id = "FlatRate"

    def __str__(self):
        """
        This is mainly helpful for debugging purposes
        """
        return "Flat Rate: %s" % config_value('SHIPPING', 'FLAT_RATE')

    def description(self):
        """
        A basic description that will be displayed to the user when selecting their shipping options
        """
        return _("Flat Rate Shipping")

    def cost(self):
        """
        Complex calculations can be done here as long as the return value is a dollar figure
        """
        assert(self._calculated)
        for cartitem in self.cart.cartitem_set.all():
            if cartitem.product.is_shippable:
                return config_value('SHIPPING', 'FLAT_RATE')
        return Decimal("0.00")

    def method(self):
        """
        Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        """
        return ugettext(config_value('SHIPPING', 'FLAT_SERVICE'))

    def expectedDelivery(self):
        """
        Can be a plain string or complex calcuation returning an actual date
        """
        return ugettext(config_value('SHIPPING', 'FLAT_DAYS'))

    def valid(self, order=None):
        """
        Can do complex validation about whether or not this option is valid.
        For example, may check to see if the recipient is in an allowed country
        or location.
        """
        return True




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2007-2019 Etienne Robillard <tkadm30@yandex.com>
All rights reserved.

This module allows communication with Schevo databases using 
the Durus 3.9 server backend.

Notes:

 1. As a minimum, You'll need the ``xdserver`` and the ``schevo``
 packages. 
 2. In addition, Durus 3.9 is being required.

"""

from schevo.backends.durus39 import XdserverBackend
from ._databaseproxy import DatabaseProxy

__all__ = ['XdserverProxy']

class XdserverProxy(DatabaseProxy):
    def __init__(self, db_name, **kwargs):
        # XXX Hack.
        if not 'db_connection_cls' in kwargs:
            kwargs['db_connection_cls'] = XdserverBackend
        super(XdserverProxy, self).__init__(db_name, **kwargs)




# -*- coding: UTF-8 -*-
"""A central mechanism forsettings which have defaults.

Repurposed from Sphene Community Tools: http://sct.sphene.net
"""

from django.conf import settings
import logging
log = logging.getLogger('l10n.l10n_settings')

l10n_settings_defaults = {
    # (or global settings) -- all other defaults should be added using
    # the add_setting_defaults method !

    'currency_formats' : {
        'USD' : {'symbol': u'$', 'positive' : u"$%(val)0.2f", 'negative': u"-$%(val)0.2f", 'decimal' : '.'},
        'GBP' : {'symbol': u'£', 'positive' : u"£%(val)0.2f", 'negative': u"-£%(val)0.2f", 'decimal' : '.'},
    },
    
    'default_currency' : 'USD',
    'show_translations': True,  #Display in the admin
    'allow_translations' : True, #Display the list of languages to store user
}


def add_setting_defaults(newdefaults):
    """
    This method can be used by other applications to define their
    default values.
    
    newdefaults has to be a dictionary containing name -> value of
    the settings.
    """
    l10n_settings_defaults.update(newdefaults)


def set_l10n_setting(name, value):
    if not hasattr(settings, 'L10N_SETTINGS'):
        settings.L10N_SETTINGS = {}
    settings.L10N_SETTINGS[name] = value
    

def get_l10n_setting(name, default_value = None):
    if not hasattr(settings, 'L10N_SETTINGS'):
        return l10n_settings_defaults.get(name, default_value)

    return settings.L10N_SETTINGS.get(name, l10n_settings_defaults.get(name, default_value))

def get_l10n_default_currency_symbol():
    key = get_l10n_setting('default_currency', default_value='USD')
    try:
        return get_l10n_setting('currency_formats')[key]['symbol']
    except KeyError:
        log.warn('could not find default currency symbol, please make sure you have a L10N_SETTINGS in your settings file, and that it has a default currency.')
        return "$"



"""Prot/X Payment Gateway.

To use this module, enable it in your shop configuration, usually at http:yourshop/settings/

To override the connection urls specified below in `PROTX_DEFAULT_URLS, add a dictionary in 
your settings.py file called "PROTX_URLS", mapping the keys below to the urls you need for 
your store.  You only need to override the specific urls that have changed, the processor
will fall back to the defaults for any not specified in your dictionary.
"""
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from payment.modules.base import BasePaymentProcessor, ProcessorResult, NOTSET
from satchmo_utils.numbers import trunc_decimal
from django.utils.http import urlencode
import forms
import urllib2

PROTOCOL = "2.22"

PROTX_DEFAULT_URLS = {
    'LIVE_CONNECTION' : 'https://ukvps.protx.com/vspgateway/service/vspdirect-register.vsp',
    'LIVE_CALLBACK' : 'https://ukvps.protx.com/vspgateway/service/direct3dcallback.vsp',
    'TEST_CONNECTION' : 'https://ukvpstest.protx.com/vspgateway/service/vspdirect-register.vsp',
    'TEST_CALLBACK' : 'https://ukvpstest.protx.com/vspgateway/service/direct3dcallback.vsp',
    'SIMULATOR_CONNECTION' : 'https://ukvpstest.protx.com/VSPSimulator/VSPDirectGateway.asp',
    'SIMULATOR_CALLBACK' : 'https://ukvpstest.protx.com/VSPSimulator/VSPDirectCallback.asp'
}

FORM = forms.ProtxPayShipForm

class PaymentProcessor(BasePaymentProcessor):
    packet = {}
    response = {}
    
    def __init__(self, settings):
        super(PaymentProcessor, self).__init__('Protx', settings)

        vendor = settings.VENDOR.value
        if vendor == "":
            self.log.warn('Prot/X Vendor is not set, please configure in your site configuration.')
        if settings.SIMULATOR.value:
            vendor = settings.VENDOR_SIMULATOR.value
            if not vendor:
                self.log.warn("You are trying to use the Prot/X VSP Simulator, but you don't have a vendor name in settings for the simulator.  I'm going to use the live vendor name, but that probably won't work.")
                vendor = settings.VENDOR.value
        
        self.packet = {
            'VPSProtocol': PROTOCOL,
            'TxType': settings.CAPTURE.value,
            'Vendor': vendor,
            'Currency': settings.CURRENCY_CODE.value,
            }
        self.valid = False

    def _url(self, key):
        urls = PROTX_DEFAULT_URLS
        if hasattr(settings, 'PROTX_URLS'):
            urls.update(settings.PROTX_URLS)

        if self.settings.SIMULATOR.value:
            key = "SIMULATOR_" + key
        else:
            if self.settings.LIVE.value:
                key = "LIVE_" + key
            else:
                key = "TEST_" + key
        return urls[key]

    def _connection(self):
        return self._url('CONNECTION')

    connection = property(fget=_connection)        
        
    def _callback(self):
        return self._url('CALLBACK')

    callback = property(fget=_callback)
        
    def prepare_post(self, data, amount):
        
        invoice = "%s" % data.id
        failct = data.paymentfailures.count()
        if failct > 0:
            invoice = "%s_%i" % (invoice, failct)
        
        try:
            cc = data.credit_card
            balance = trunc_decimal(data.balance, 2)
            self.packet['VendorTxCode'] = invoice
            self.packet['Amount'] = balance
            self.packet['Description'] = 'Online purchase'
            self.packet['CardType'] = cc.credit_type
            self.packet['CardHolder'] = cc.card_holder
            self.packet['CardNumber'] = cc.decryptedCC
            self.packet['ExpiryDate'] = '%02d%s' % (cc.expire_month, str(cc.expire_year)[2:])
            if cc.start_month is not None:
                self.packet['StartDate'] = '%02d%s' % (cc.start_month, str(cc.start_year)[2:])
            if cc.ccv is not None and cc.ccv != "":
                self.packet['CV2'] = cc.ccv
            if cc.issue_num is not None and cc.issue_num != "":
                self.packet['IssueNumber'] = cc.issue_num #'%02d' % int(cc.issue_num)
            addr = [data.bill_street1, data.bill_street2, data.bill_city, data.bill_state]
            self.packet['BillingAddress'] = ', '.join(addr)
            self.packet['BillingPostCode'] = data.bill_postal_code
        except Exception, e:
            self.log.error('preparing data, got error: %s\nData: %s', e, data)
            self.valid = False
            return
            
        # handle pesky unicode chars in names
        for key, value in self.packet.items():
            try:
                value = value.encode('utf-8')
                self.packet[key] = value
            except AttributeError:
                pass
        
        self.postString = urlencode(self.packet)
        self.url = self.connection
        self.valid = True
    
    def prepare_data3d(self, md, pares):
        self.packet = {}
        self.packet['MD'] = md
        self.packet['PARes'] = pares
        self.postString = urlencode(self.packet)
        self.url = self.callback
        self.valid = True
        
    def capture_payment(self, testing=False, order=None, amount=NOTSET):
        """Execute the post to protx VSP DIRECT"""
        if not order:
            order = self.order

        if order.paid_in_full:
            self.log_extra('%s is paid in full, no capture attempted.', order)
            self.record_payment()
            return ProcessorResult(self.key, True, _("No charge needed, paid in full."))

        self.log_extra('Capturing payment for %s', order)

        if amount == NOTSET:
            amount = order.balance

        self.prepare_post(order, amount)
        
        if self.valid:
            if self.settings.SKIP_POST.value:
                self.log.info("TESTING MODE - Skipping post to server.  Would have posted %s?%s", self.url, self.postString)
                payment = self.record_payment(order=order, amount=amount, 
                    transaction_id="TESTING", reason_code='0')

                return ProcessorResult(self.key, True, _('TESTING MODE'), payment=payment)
                
            else:
                self.log_extra("About to post to server: %s?%s", self.url, self.postString)
                conn = urllib2.Request(self.url, data=self.postString)
                try:
                    f = urllib2.urlopen(conn)
                    result = f.read()
                    self.log_extra('Process: url=%s\nPacket=%s\nResult=%s', self.url, self.packet, result)

                except urllib2.URLError, ue:
                    self.log.error("error opening %s\n%s", self.url, ue)
                    return (False, 'ERROR', 'Could not talk to Protx gateway')

                try:
                    self.response = dict([row.split('=', 1) for row in result.splitlines()])
                    status = self.response['Status']
                    success = (status == 'OK')
                    detail = self.response['StatusDetail']
                    
                    payment = None
                    transaction_id = ""
                    if success:
                        vpstxid = self.response.get('VPSTxID', '')
                        txauthno = self.response.get('TxAuthNo', '')
                        transaction_id="%s,%s" % (vpstxid, txauthno)
                        self.log.info('Success on order #%i, recording payment', self.order.id)
                        payment = self.record_payment(order=order, amount=amount, 
                            transaction_id=transaction_id, reason_code=status)
                            
                    else:
                        payment = self.record_failure(order=order, amount=amount, 
                            transaction_id=transaction_id, reason_code=status, 
                            details=detail)

                    return ProcessorResult(self.key, success, detail, payment=payment)

                except Exception, e:
                    self.log.info('Error submitting payment: %s', e)
                    payment = self.record_failure(order=order, amount=amount, 
                        transaction_id="", reason_code="error", 
                        details='Invalid response from payment gateway')
                    
                    return ProcessorResult(self.key, False, _('Invalid response from payment gateway'))
        else:
            return ProcessorResult(self.key, False, _('Error processing payment.'))



"""
GiftCertificate processor
"""
from django.utils.translation import ugettext as _
from l10n.utils import moneyfmt
from models import GiftCertificate
from payment.modules.base import BasePaymentProcessor, ProcessorResult, NOTSET

class PaymentProcessor(BasePaymentProcessor):

    def __init__(self, settings):
        super(PaymentProcessor, self).__init__('giftcertificate', settings)

    def capture_payment(self, testing=False, order=None, amount=NOTSET):
        """
        Process the transaction and return a ProcessorResponse
        """
        if not order:
            order = self.order
            
        if amount==NOTSET:
            amount = order.balance

        payment = None

        valid_gc = False
        if self.order.paid_in_full:
            success = True
            reason_code = "0"
            response_text = _("No balance to pay")
            self.record_payment()

        else:
            try:
                gc = GiftCertificate.objects.from_order(self.order)
                valid_gc = gc.valid

            except GiftCertificate.DoesNotExist:
                success = False
                reason_code="1"
                response_text = _("No such Gift Certificate")

            if not valid_gc:
                success = False
                reason_code="2"
                response_text = _("Bad Gift Certificate")

            else:
                gc.apply_to_order(self.order)
                payment = gc.orderpayment
                reason_code = "0"
                response_text = _("Success")
                success = True

                if not self.order.paid_in_full:
                    response_text = _("%s balance remains after gift certificate was applied") % moneyfmt(self.order.balance)

        return ProcessorResult(self.key, success, response_text, payment=payment)



from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse as url
from django.test import TestCase
from django.test.client import Client
from django.utils.encoding import smart_str
from keyedcache import cache_delete
from l10n.models import Country
from livesettings import config_value, config_get
from product.models import Product
from product.utils import rebuild_pricing
from satchmo_ext.wishlist.models import *
from satchmo_store.contact.models import Contact
from satchmo_utils.templatetags import get_filter_args

domain = 'http://testserver'

def get_step1_post_data(US):
    return {
        'email': 'sometester@example.com',
        'first_name': 'Teddy',
        'last_name' : 'Tester',
        'phone': '456-123-5555',
        'street1': '8299 Some Street',
        'city': 'Springfield',
        'state': 'MO',
        'postal_code': '81122',
        'country': US.pk,
        'ship_street1': '1011 Some Other Street',
        'ship_city': 'Springfield',
        'ship_state': 'MO',
        'ship_postal_code': '81123',
        'paymentmethod': 'DUMMY'
    }

class WishTest(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'test-config.yaml']

    def setUp(self):
        # Every test needs a client
        self.client = Client()
        rebuild_pricing()

    def tearDown(self):
        cache_delete()

    def test_main_page(self):
        """
        Look at the main page
        """
        response = self.client.get(url('satchmo_shop_home'))

        # Check that the rendered context contains 4 products
        self.assertContains(response, '<div class = "productImage">',
                            count=4, status_code=200)

    def test_wish_adding_not_logged_in(self):
        """
        Validate we can't add unless we are logged in.
        """
        product = Product.objects.get(slug='dj-rocks')
        response = self.client.get(product.get_absolute_url())
        self.assertContains(response, "Django Rocks shirt", count=2, status_code=200)
        response = self.client.post(url('satchmo_smart_add'), { 
            "productname" : "dj-rocks",
            "1" : "M",
            "2" : "BL",
            "addwish" : "Add to wishlist"
        })
        self.assertContains(response, "Sorry, you must be", count=1, status_code=200)
    
class WishTestLoggedIn(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'test-config.yaml']
    
    def setUp(self):
        self.client = Client()
        user = User.objects.create_user('wisher', 'wisher@example.com', 'passwd')
        user.is_staff = False
        user.is_superuser = False
        user.save()
        self.contact = Contact.objects.create(user=user, first_name="Wish",
            last_name="Tester")
        self.client.login(username='wisher', password='passwd')
        rebuild_pricing()
        
    def tearDown(self):
        cache_delete()
        
    def test_wish_adding(self):
        """
        Validate we can add some items to the wishlist
        """
        product = Product.objects.get(slug='dj-rocks')
        response = self.client.get(product.get_absolute_url())
        self.assertContains(response, "Django Rocks shirt", count=2, status_code=200)
        response = self.client.post(url('satchmo_smart_add'), { 
            "productname" : "dj-rocks",
            "1" : "M",
            "2" : "BL",
            "addwish" : "Add to wishlist"
        })
        wishurl = url('satchmo_wishlist_view')
        self.assertRedirects(response, domain + wishurl, status_code=302, target_status_code=200)
        response = self.client.get(wishurl)
        
        self.assertContains(response, "Django Rocks shirt (Medium/Blue)", count=1, status_code=200)

    def test_wish_removing(self):
        """
        Validate that we can remove wishlist items
        """
        product = Product.objects.get(slug="dj-rocks-m-bl")
        wish = ProductWish(product = product, contact=self.contact)
        wish.save()
        
        product = Product.objects.get(slug="robot-attack-soft")
        wish = ProductWish(product = product, contact=self.contact)
        wish.save()
        
        wishurl = url('satchmo_wishlist_view')
        response = self.client.get(wishurl)
        self.assertContains(response, "Robots Attack", count=1, status_code=200)
        self.assertContains(response, "Django Rocks shirt (Medium/Blue)", count=1, status_code=200)
        
        wishurl = url('satchmo_wishlist_remove')
        response = self.client.post(wishurl, {
            'id' : wish.id
        })
        self.assertContains(response, "Robots Attack", count=0, status_code=200)
        self.assertContains(response, "Django Rocks shirt (Medium/Blue)", count=1, status_code=200)

    def test_wish_to_cart(self):
        """
        Validate that we can move an item to the cart
        """
        product = Product.objects.get(slug="dj-rocks-m-bl")
        wish = ProductWish(product = product, contact=self.contact)
        wish.save()
        
        product = Product.objects.get(slug="robot-attack-soft")
        wish = ProductWish(product = product, contact=self.contact)
        wish.save()
        
        wishurl = url('satchmo_wishlist_view')
        response = self.client.get(wishurl)
        self.assertContains(response, "Robots Attack", count=1, status_code=200)
        self.assertContains(response, "Django Rocks shirt (Medium/Blue)", count=1, status_code=200)
        
        moveurl = url('satchmo_wishlist_move_to_cart')
        response = self.client.post(moveurl, {
            'id' : wish.id
        })
        carturl = url('satchmo_cart')
        self.assertRedirects(response, domain + carturl, status_code=302, target_status_code=200)

        response = self.client.get(carturl)
        self.assertContains(response, "Robots Attack", count=1, status_code=200)
        
        


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):

        # Adding model 'Category'
        db.create_table('product_category', (
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(related_name='child', blank=True, null=True, to=orm['product.Category'])),
            ('ordering', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('is_active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('meta', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(db_index=True, max_length=50, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal('product', ['Category'])

        # Adding unique constraint on 'Category', fields ['site', 'slug']
        db.create_unique('product_category', ['site_id', 'slug'])

        # Adding M2M table for field related_categories on 'Category'
        db.create_table('product_category_related_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('from_category', models.ForeignKey(orm['product.category'], null=False)),
            ('to_category', models.ForeignKey(orm['product.category'], null=False))
        ))
        db.create_unique('product_category_related_categories', ['from_category_id', 'to_category_id'])

        # Adding model 'CategoryTranslation'
        db.create_table('product_categorytranslation', (
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['product.Category'])),
            ('languagecode', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('description', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('version', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('product', ['CategoryTranslation'])

        # Adding unique constraint on 'CategoryTranslation', fields ['category', 'languagecode', 'version']
        db.create_unique('product_categorytranslation', ['category_id', 'languagecode', 'version'])

        # Adding model 'CategoryImage'
        db.create_table('product_categoryimage', (
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(related_name='images', blank=True, null=True, to=orm['product.Category'])),
            ('picture', self.gf('satchmo_utils.thumbnail.field.ImageWithThumbnailField')(name_field='_filename', max_length=200)),
            ('sort', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('caption', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
        ))
        db.send_create_signal('product', ['CategoryImage'])

        # Adding unique constraint on 'CategoryImage', fields ['category', 'sort']
        db.create_unique('product_categoryimage', ['category_id', 'sort'])

        # Adding model 'CategoryImageTranslation'
        db.create_table('product_categoryimagetranslation', (
            ('languagecode', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('categoryimage', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['product.CategoryImage'])),
            ('caption', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('version', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('product', ['CategoryImageTranslation'])

        # Adding unique constraint on 'CategoryImageTranslation', fields ['categoryimage', 'languagecode', 'version']
        db.create_unique('product_categoryimagetranslation', ['categoryimage_id', 'languagecode', 'version'])

        # Adding model 'Discount'
        db.create_table('product_discount', (
            ('startDate', self.gf('django.db.models.fields.DateField')()),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=20, unique=True)),
            ('endDate', self.gf('django.db.models.fields.DateField')()),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('numUses', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('allowedUses', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('shipping', self.gf('django.db.models.fields.CharField')(default='NONE', max_length=10, null=True, blank=True)),
            ('allValid', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('amount', self.gf('satchmo_utils.fields.CurrencyField')(null=True, max_digits=8, decimal_places=2, blank=True)),
            ('minOrder', self.gf('satchmo_utils.fields.CurrencyField')(null=True, max_digits=8, decimal_places=2, blank=True)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('percentage', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=5, decimal_places=2, blank=True)),
            ('automatic', self.gf('django.db.models.fields.NullBooleanField')(default=False, null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('product', ['Discount'])

        # Adding M2M table for field validProducts on 'Discount'
        db.create_table('product_discount_validProducts', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('discount', models.ForeignKey(orm['product.discount'], null=False)),
            ('product', models.ForeignKey(orm['product.product'], null=False))
        ))
        db.create_unique('product_discount_validProducts', ['discount_id', 'product_id'])

        # Adding model 'OptionGroup'
        db.create_table('product_optiongroup', (
            ('description', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('sort_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
        ))
        db.send_create_signal('product', ['OptionGroup'])

        # Adding model 'OptionGroupTranslation'
        db.create_table('product_optiongrouptranslation', (
            ('languagecode', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('description', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('version', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('optiongroup', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['product.OptionGroup'])),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('product', ['OptionGroupTranslation'])

        # Adding unique constraint on 'OptionGroupTranslation', fields ['optiongroup', 'languagecode', 'version']
        db.create_unique('product_optiongrouptranslation', ['optiongroup_id', 'languagecode', 'version'])

        # Adding model 'Option'
        db.create_table('product_option', (
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('price_change', self.gf('satchmo_utils.fields.CurrencyField')(null=True, max_digits=14, decimal_places=6, blank=True)),
            ('option_group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['product.OptionGroup'])),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('sort_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('product', ['Option'])

        # Adding unique constraint on 'Option', fields ['option_group', 'value']
        db.create_unique('product_option', ['option_group_id', 'value'])

        # Adding model 'OptionTranslation'
        db.create_table('product_optiontranslation', (
            ('languagecode', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('version', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('option', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['product.Option'])),
        ))
        db.send_create_signal('product', ['OptionTranslation'])

        # Adding unique constraint on 'OptionTranslation', fields ['option', 'languagecode', 'version']
        db.create_unique('product_optiontranslation', ['option_id', 'languagecode', 'version'])

        # Adding model 'Product'
        db.create_table('product_product', (
            ('weight', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=8, decimal_places=2, blank=True)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('height', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=6, decimal_places=2, blank=True)),
            ('taxable', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('featured', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('meta', self.gf('django.db.models.fields.TextField')(max_length=200, null=True, blank=True)),
            ('total_sold', self.gf('django.db.models.fields.DecimalField')(default='0', max_digits=18, decimal_places=6)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('sku', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('taxClass', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['product.TaxClass'], null=True, blank=True)),
            ('ordering', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('height_units', self.gf('django.db.models.fields.CharField')(max_length=3, null=True, blank=True)),
            ('width', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=6, decimal_places=2, blank=True)),
            ('items_in_stock', self.gf('django.db.models.fields.DecimalField')(default='0', max_digits=18, decimal_places=6)),
            ('short_description', self.gf('django.db.models.fields.TextField')(default='', max_length=200, blank=True)),
            ('length_units', self.gf('django.db.models.fields.CharField')(max_length=3, null=True, blank=True)),
            ('weight_units', self.gf('django.db.models.fields.CharField')(max_length=3, null=True, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('date_added', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(db_index=True, max_length=255, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('length', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=6, decimal_places=2, blank=True)),
            ('shipclass', self.gf('django.db.models.fields.CharField')(default='DEFAULT', max_length=10)),
            ('width_units', self.gf('django.db.models.fields.CharField')(max_length=3, null=True, blank=True)),
        ))
        db.send_create_signal('product', ['Product'])

        # Adding unique constraint on 'Product', fields ['site', 'sku']
        db.create_unique('product_product', ['site_id', 'sku'])

        # Adding unique constraint on 'Product', fields ['site', 'slug']
        db.create_unique('product_product', ['site_id', 'slug'])

        # Adding M2M table for field category on 'Product'
        db.create_table('product_product_category', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('product', models.ForeignKey(orm['product.product'], null=False)),
            ('category', models.ForeignKey(orm['product.category'], null=False))
        ))
        db.create_unique('product_product_category', ['product_id', 'category_id'])

        # Adding M2M table for field also_purchased on 'Product'
        db.create_table('product_product_also_purchased', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('from_product', models.ForeignKey(orm['product.product'], null=False)),
            ('to_product', models.ForeignKey(orm['product.product'], null=False))
        ))
        db.create_unique('product_product_also_purchased', ['from_product_id', 'to_product_id'])

        # Adding M2M table for field related_items on 'Product'
        db.create_table('product_product_related_items', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('from_product', models.ForeignKey(orm['product.product'], null=False)),
            ('to_product', models.ForeignKey(orm['product.product'], null=False))
        ))
        db.create_unique('product_product_related_items', ['from_product_id', 'to_product_id'])

        # Adding model 'ProductTranslation'
        db.create_table('product_producttranslation', (
            ('languagecode', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('product', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['product.Product'])),
            ('description', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('version', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('short_description', self.gf('django.db.models.fields.TextField')(default='', max_length=200, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('product', ['ProductTranslation'])

        # Adding unique constraint on 'ProductTranslation', fields ['product', 'languagecode', 'version']
        db.create_unique('product_producttranslation', ['product_id', 'languagecode', 'version'])

        # Adding model 'CustomProduct'
        db.create_table('product_customproduct', (
            ('product', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['product.Product'], unique=True, primary_key=True)),
            ('deferred_shipping', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('downpayment', self.gf('django.db.models.fields.IntegerField')(default=20)),
        ))
        db.send_create_signal('product', ['CustomProduct'])

        # Adding M2M table for field option_group on 'CustomProduct'
        db.create_table('product_customproduct_option_group', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('customproduct', models.ForeignKey(orm['product.customproduct'], null=False)),
            ('optiongroup', models.ForeignKey(orm['product.optiongroup'], null=False))
        ))
        db.create_unique('product_customproduct_option_group', ['customproduct_id', 'optiongroup_id'])

        # Adding model 'CustomTextField'
        db.create_table('product_customtextfield', (
            ('name', self.gf('django.db.models.fields.CharField')(max_length=40)),
            ('price_change', self.gf('satchmo_utils.fields.CurrencyField')(null=True, max_digits=14, decimal_places=6, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('sort_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('products', self.gf('django.db.models.fields.related.ForeignKey')(related_name='custom_text_fields', to=orm['product.CustomProduct'])),
            ('slug', self.gf('django.db.models.fields.SlugField')(db_index=True, max_length=50, blank=True)),
        ))
        db.send_create_signal('product', ['CustomTextField'])

        # Adding model 'CustomTextFieldTranslation'
        db.create_table('product_customtextfieldtranslation', (
            ('languagecode', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('version', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('customtextfield', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['product.CustomTextField'])),
        ))
        db.send_create_signal('product', ['CustomTextFieldTranslation'])

        # Adding unique constraint on 'CustomTextFieldTranslation', fields ['customtextfield', 'languagecode', 'version']
        db.create_unique('product_customtextfieldtranslation', ['customtextfield_id', 'languagecode', 'version'])

        # Adding model 'ConfigurableProduct'
        db.create_table('product_configurableproduct', (
            ('create_subs', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('product', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['product.Product'], unique=True, primary_key=True)),
        ))
        db.send_create_signal('product', ['ConfigurableProduct'])

        # Adding M2M table for field option_group on 'ConfigurableProduct'
        db.create_table('product_configurableproduct_option_group', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('configurableproduct', models.ForeignKey(orm['product.configurableproduct'], null=False)),
            ('optiongroup', models.ForeignKey(orm['product.optiongroup'], null=False))
        ))
        db.create_unique('product_configurableproduct_option_group', ['configurableproduct_id', 'optiongroup_id'])

        # Adding model 'DownloadableProduct'
        db.create_table('product_downloadableproduct', (
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('product', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['product.Product'], unique=True, primary_key=True)),
            ('num_allowed_downloads', self.gf('django.db.models.fields.IntegerField')()),
            ('file', self.gf('django.db.models.fields.files.FileField')(max_length=100)),
            ('expire_minutes', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('product', ['DownloadableProduct'])

        # Adding model 'SubscriptionProduct'
        db.create_table('product_subscriptionproduct', (
            ('product', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['product.Product'], unique=True, primary_key=True)),
            ('is_shippable', self.gf('django.db.models.fields.IntegerField')(max_length=1)),
            ('recurring_times', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('expire_unit', self.gf('django.db.models.fields.CharField')(default='DAY', max_length=5)),
            ('expire_length', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('recurring', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
        ))
        db.send_create_signal('product', ['SubscriptionProduct'])

        # Adding model 'Trial'
        db.create_table('product_trial', (
            ('expire_length', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('price', self.gf('satchmo_utils.fields.CurrencyField')(null=True, max_digits=10, decimal_places=2)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('subscription', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['product.SubscriptionProduct'])),
        ))
        db.send_create_signal('product', ['Trial'])

        # Adding model 'ProductVariation'
        db.create_table('product_productvariation', (
            ('product', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['product.Product'], unique=True, primary_key=True)),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['product.ConfigurableProduct'])),
        ))
        db.send_create_signal('product', ['ProductVariation'])

        # Adding M2M table for field options on 'ProductVariation'
        db.create_table('product_productvariation_options', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('productvariation', models.ForeignKey(orm['product.productvariation'], null=False)),
            ('option', models.ForeignKey(orm['product.option'], null=False))
        ))
        db.create_unique('product_productvariation_options', ['productvariation_id', 'option_id'])

        # Adding model 'ProductPriceLookup'
        db.create_table('product_productpricelookup', (
            ('discountable', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('price', self.gf('django.db.models.fields.DecimalField')(max_digits=14, decimal_places=6)),
            ('items_in_stock', self.gf('django.db.models.fields.DecimalField')(max_digits=18, decimal_places=6)),
            ('siteid', self.gf('django.db.models.fields.IntegerField')()),
            ('key', self.gf('django.db.models.fields.CharField')(max_length=60, null=True)),
            ('parentid', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('productslug', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('quantity', self.gf('django.db.models.fields.DecimalField')(max_digits=18, decimal_places=6)),
        ))
        db.send_create_signal('product', ['ProductPriceLookup'])

        # Adding model 'ProductAttribute'
        db.create_table('product_productattribute', (
            ('languagecode', self.gf('django.db.models.fields.CharField')(max_length=10, null=True, blank=True)),
            ('product', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['product.Product'])),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('name', self.gf('django.db.models.fields.SlugField')(max_length=100, db_index=True)),
        ))
        db.send_create_signal('product', ['ProductAttribute'])

        # Adding model 'Price'
        db.create_table('product_price', (
            ('price', self.gf('satchmo_utils.fields.CurrencyField')(max_digits=14, decimal_places=6)),
            ('product', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['product.Product'])),
            ('expires', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('quantity', self.gf('django.db.models.fields.DecimalField')(default='1.0', max_digits=18, decimal_places=6)),
        ))
        db.send_create_signal('product', ['Price'])

        # Adding unique constraint on 'Price', fields ['product', 'quantity', 'expires']
        db.create_unique('product_price', ['product_id', 'quantity', 'expires'])

        # Adding model 'ProductImage'
        db.create_table('product_productimage', (
            ('sort', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('picture', self.gf('satchmo_utils.thumbnail.field.ImageWithThumbnailField')(name_field='_filename', max_length=200)),
            ('product', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['product.Product'], null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('caption', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
        ))
        db.send_create_signal('product', ['ProductImage'])

        # Adding model 'ProductImageTranslation'
        db.create_table('product_productimagetranslation', (
            ('languagecode', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('productimage', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['product.ProductImage'])),
            ('caption', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('version', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('product', ['ProductImageTranslation'])

        # Adding unique constraint on 'ProductImageTranslation', fields ['productimage', 'languagecode', 'version']
        db.create_unique('product_productimagetranslation', ['productimage_id', 'languagecode', 'version'])

        # Adding model 'TaxClass'
        db.create_table('product_taxclass', (
            ('description', self.gf('django.db.models.fields.CharField')(max_length=30)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=20)),
        ))
        db.send_create_signal('product', ['TaxClass'])


    def backwards(self, orm):

        # Deleting model 'Category'
        db.delete_table('product_category')

        # Removing unique constraint on 'Category', fields ['site', 'slug']
        db.delete_unique('product_category', ['site_id', 'slug'])

        # Removing M2M table for field related_categories on 'Category'
        db.delete_table('product_category_related_categories')

        # Deleting model 'CategoryTranslation'
        db.delete_table('product_categorytranslation')

        # Removing unique constraint on 'CategoryTranslation', fields ['category', 'languagecode', 'version']
        db.delete_unique('product_categorytranslation', ['category_id', 'languagecode', 'version'])

        # Deleting model 'CategoryImage'
        db.delete_table('product_categoryimage')

        # Removing unique constraint on 'CategoryImage', fields ['category', 'sort']
        db.delete_unique('product_categoryimage', ['category_id', 'sort'])

        # Deleting model 'CategoryImageTranslation'
        db.delete_table('product_categoryimagetranslation')

        # Removing unique constraint on 'CategoryImageTranslation', fields ['categoryimage', 'languagecode', 'version']
        db.delete_unique('product_categoryimagetranslation', ['categoryimage_id', 'languagecode', 'version'])

        # Deleting model 'Discount'
        db.delete_table('product_discount')

        # Removing M2M table for field validProducts on 'Discount'
        db.delete_table('product_discount_validProducts')

        # Deleting model 'OptionGroup'
        db.delete_table('product_optiongroup')

        # Deleting model 'OptionGroupTranslation'
        db.delete_table('product_optiongrouptranslation')

        # Removing unique constraint on 'OptionGroupTranslation', fields ['optiongroup', 'languagecode', 'version']
        db.delete_unique('product_optiongrouptranslation', ['optiongroup_id', 'languagecode', 'version'])

        # Deleting model 'Option'
        db.delete_table('product_option')

        # Removing unique constraint on 'Option', fields ['option_group', 'value']
        db.delete_unique('product_option', ['option_group_id', 'value'])

        # Deleting model 'OptionTranslation'
        db.delete_table('product_optiontranslation')

        # Removing unique constraint on 'OptionTranslation', fields ['option', 'languagecode', 'version']
        db.delete_unique('product_optiontranslation', ['option_id', 'languagecode', 'version'])

        # Deleting model 'Product'
        db.delete_table('product_product')

        # Removing unique constraint on 'Product', fields ['site', 'sku']
        db.delete_unique('product_product', ['site_id', 'sku'])

        # Removing unique constraint on 'Product', fields ['site', 'slug']
        db.delete_unique('product_product', ['site_id', 'slug'])

        # Removing M2M table for field category on 'Product'
        db.delete_table('product_product_category')

        # Removing M2M table for field also_purchased on 'Product'
        db.delete_table('product_product_also_purchased')

        # Removing M2M table for field related_items on 'Product'
        db.delete_table('product_product_related_items')

        # Deleting model 'ProductTranslation'
        db.delete_table('product_producttranslation')

        # Removing unique constraint on 'ProductTranslation', fields ['product', 'languagecode', 'version']
        db.delete_unique('product_producttranslation', ['product_id', 'languagecode', 'version'])

        # Deleting model 'CustomProduct'
        db.delete_table('product_customproduct')

        # Removing M2M table for field option_group on 'CustomProduct'
        db.delete_table('product_customproduct_option_group')

        # Deleting model 'CustomTextField'
        db.delete_table('product_customtextfield')

        # Deleting model 'CustomTextFieldTranslation'
        db.delete_table('product_customtextfieldtranslation')

        # Removing unique constraint on 'CustomTextFieldTranslation', fields ['customtextfield', 'languagecode', 'version']
        db.delete_unique('product_customtextfieldtranslation', ['customtextfield_id', 'languagecode', 'version'])

        # Deleting model 'ConfigurableProduct'
        db.delete_table('product_configurableproduct')

        # Removing M2M table for field option_group on 'ConfigurableProduct'
        db.delete_table('product_configurableproduct_option_group')

        # Deleting model 'DownloadableProduct'
        db.delete_table('product_downloadableproduct')

        # Deleting model 'SubscriptionProduct'
        db.delete_table('product_subscriptionproduct')

        # Deleting model 'Trial'
        db.delete_table('product_trial')

        # Deleting model 'ProductVariation'
        db.delete_table('product_productvariation')

        # Removing M2M table for field options on 'ProductVariation'
        db.delete_table('product_productvariation_options')

        # Deleting model 'ProductPriceLookup'
        db.delete_table('product_productpricelookup')

        # Deleting model 'ProductAttribute'
        db.delete_table('product_productattribute')

        # Deleting model 'Price'
        db.delete_table('product_price')

        # Removing unique constraint on 'Price', fields ['product', 'quantity', 'expires']
        db.delete_unique('product_price', ['product_id', 'quantity', 'expires'])

        # Deleting model 'ProductImage'
        db.delete_table('product_productimage')

        # Deleting model 'ProductImageTranslation'
        db.delete_table('product_productimagetranslation')

        # Removing unique constraint on 'ProductImageTranslation', fields ['productimage', 'languagecode', 'version']
        db.delete_unique('product_productimagetranslation', ['productimage_id', 'languagecode', 'version'])

        # Deleting model 'TaxClass'
        db.delete_table('product_taxclass')


    models = {
        'product.category': {
            'Meta': {'unique_together': "(('site', 'slug'),)", 'object_name': 'Category'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'meta': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'child'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'related_categories': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'related_categories'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '50', 'blank': 'True'})
        },
        'product.categoryimage': {
            'Meta': {'unique_together': "(('category', 'sort'),)", 'object_name': 'CategoryImage'},
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'images'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'picture': ('satchmo_utils.thumbnail.field.ImageWithThumbnailField', [], {'name_field': "'_filename'", 'max_length': '200'}),
            'sort': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.categoryimagetranslation': {
            'Meta': {'unique_together': "(('categoryimage', 'languagecode', 'version'),)", 'object_name': 'CategoryImageTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'categoryimage': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.CategoryImage']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.categorytranslation': {
            'Meta': {'unique_together': "(('category', 'languagecode', 'version'),)", 'object_name': 'CategoryTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.Category']"}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.configurableproduct': {
            'Meta': {'object_name': 'ConfigurableProduct'},
            'create_subs': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'option_group': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.OptionGroup']", 'blank': 'True'}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'product.customproduct': {
            'Meta': {'object_name': 'CustomProduct'},
            'deferred_shipping': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'downpayment': ('django.db.models.fields.IntegerField', [], {'default': '20'}),
            'option_group': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.OptionGroup']", 'blank': 'True'}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'product.customtextfield': {
            'Meta': {'object_name': 'CustomTextField'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'price_change': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '14', 'decimal_places': '6', 'blank': 'True'}),
            'products': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'custom_text_fields'", 'to': "orm['product.CustomProduct']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '50', 'blank': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.customtextfieldtranslation': {
            'Meta': {'unique_together': "(('customtextfield', 'languagecode', 'version'),)", 'object_name': 'CustomTextFieldTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'customtextfield': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.CustomTextField']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.discount': {
            'Meta': {'object_name': 'Discount'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'allValid': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'allowedUses': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'amount': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'automatic': ('django.db.models.fields.NullBooleanField', [], {'default': 'False', 'null': 'True', 'blank': 'True'}),
            'code': ('django.db.models.fields.CharField', [], {'max_length': '20', 'unique': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'endDate': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'minOrder': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'numUses': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'percentage': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '5', 'decimal_places': '2', 'blank': 'True'}),
            'shipping': ('django.db.models.fields.CharField', [], {'default': "'NONE'", 'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'startDate': ('django.db.models.fields.DateField', [], {}),
            'validProducts': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.Product']", 'null': 'True', 'blank': 'True'})
        },
        'product.downloadableproduct': {
            'Meta': {'object_name': 'DownloadableProduct'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'expire_minutes': ('django.db.models.fields.IntegerField', [], {}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'num_allowed_downloads': ('django.db.models.fields.IntegerField', [], {}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'product.option': {
            'Meta': {'unique_together': "(('option_group', 'value'),)", 'object_name': 'Option'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'option_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.OptionGroup']"}),
            'price_change': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '14', 'decimal_places': '6', 'blank': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'product.optiongroup': {
            'Meta': {'object_name': 'OptionGroup'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.optiongrouptranslation': {
            'Meta': {'unique_together': "(('optiongroup', 'languagecode', 'version'),)", 'object_name': 'OptionGroupTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'optiongroup': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.OptionGroup']"}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.optiontranslation': {
            'Meta': {'unique_together': "(('option', 'languagecode', 'version'),)", 'object_name': 'OptionTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'option': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.Option']"}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.price': {
            'Meta': {'unique_together': "(('product', 'quantity', 'expires'),)", 'object_name': 'Price'},
            'expires': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'price': ('satchmo_utils.fields.CurrencyField', [], {'max_digits': '14', 'decimal_places': '6'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.Product']"}),
            'quantity': ('django.db.models.fields.DecimalField', [], {'default': "'1.0'", 'max_digits': '18', 'decimal_places': '6'})
        },
        'product.product': {
            'Meta': {'unique_together': "(('site', 'sku'), ('site', 'slug'))", 'object_name': 'Product'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'also_purchased': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'also_products'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"}),
            'category': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.Category']", 'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'featured': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'height': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'height_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'items_in_stock': ('django.db.models.fields.DecimalField', [], {'default': "'0'", 'max_digits': '18', 'decimal_places': '6'}),
            'length': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'length_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'meta': ('django.db.models.fields.TextField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'related_items': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'related_products'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"}),
            'shipclass': ('django.db.models.fields.CharField', [], {'default': "'DEFAULT'", 'max_length': '10'}),
            'short_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'sku': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '255', 'blank': 'True'}),
            'taxClass': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.TaxClass']", 'null': 'True', 'blank': 'True'}),
            'taxable': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'total_sold': ('django.db.models.fields.DecimalField', [], {'default': "'0'", 'max_digits': '18', 'decimal_places': '6'}),
            'weight': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'weight_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'width': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'width_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'})
        },
        'product.productattribute': {
            'Meta': {'object_name': 'ProductAttribute'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.SlugField', [], {'max_length': '100', 'db_index': 'True'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.Product']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'product.productimage': {
            'Meta': {'object_name': 'ProductImage'},
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'picture': ('satchmo_utils.thumbnail.field.ImageWithThumbnailField', [], {'name_field': "'_filename'", 'max_length': '200'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.Product']", 'null': 'True', 'blank': 'True'}),
            'sort': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.productimagetranslation': {
            'Meta': {'unique_together': "(('productimage', 'languagecode', 'version'),)", 'object_name': 'ProductImageTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'productimage': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.ProductImage']"}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.productpricelookup': {
            'Meta': {'object_name': 'ProductPriceLookup'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'discountable': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'items_in_stock': ('django.db.models.fields.DecimalField', [], {'max_digits': '18', 'decimal_places': '6'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '60', 'null': 'True'}),
            'parentid': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '14', 'decimal_places': '6'}),
            'productslug': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'quantity': ('django.db.models.fields.DecimalField', [], {'max_digits': '18', 'decimal_places': '6'}),
            'siteid': ('django.db.models.fields.IntegerField', [], {})
        },
        'product.producttranslation': {
            'Meta': {'unique_together': "(('product', 'languagecode', 'version'),)", 'object_name': 'ProductTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.Product']"}),
            'short_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.productvariation': {
            'Meta': {'object_name': 'ProductVariation'},
            'options': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.Option']"}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.ConfigurableProduct']"}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'product.subscriptionproduct': {
            'Meta': {'object_name': 'SubscriptionProduct'},
            'expire_length': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'expire_unit': ('django.db.models.fields.CharField', [], {'default': "'DAY'", 'max_length': '5'}),
            'is_shippable': ('django.db.models.fields.IntegerField', [], {'max_length': '1'}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'}),
            'recurring': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'recurring_times': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'product.taxclass': {
            'Meta': {'object_name': 'TaxClass'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '30'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        'product.trial': {
            'Meta': {'object_name': 'Trial'},
            'expire_length': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'price': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '10', 'decimal_places': '2'}),
            'subscription': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.SubscriptionProduct']"})
        },
        'sites.site': {
            'Meta': {'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['product']



#!/usr/bin/env python

import asyncio
from concurrent.futures import ThreadPoolExecutor

import logging
log = logging.getLogger('asyncio')
log.setLevel(logging.DEBUG)

from wsgiref import asyncio_server
from notmm.controllers.async import AsyncIOController
from notmm.utils.wsgilib import HTTPResponse, HTTPRequest

def start_loop(threads=4):
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    assert threads >= 1, "threads should be >= 1"
    executor = ThreadPoolExecutor(threads)
    return loop, executor

@asyncio.coroutine
def helloworld(environ, start_response):
    return (yield from HTTPResponse("hello world")(environ, start_response))

@asyncio.coroutine
def application(environ, start_response):
    request = HTTPRequest(environ)
    try:
        result = (yield from AsyncIOController()(environ, start_response))
    except:
        raise
    else:
        yield from result

if __name__ == '__main__':
    loop, executor = start_loop()
    
    server = asyncio_server.make_server('127.0.0.1', 8000, application)
    loop.run_in_executor(executor, server.serve_forever())




from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _, ugettext
from satchmo_ext.newsletter.forms import NewsletterForm

def add_subscription(request, template="newsletter/subscribe_form.html", 
    result_template="newsletter/update_results.html", form=NewsletterForm):
    """Add a subscription and return the results in the requested template."""

    return _update(request, True, template, result_template, form=form)

def remove_subscription(request, template="newsletter/unsubscribe_form.html",
    result_template="newsletter/update_results.html", form=NewsletterForm):
    """Remove a subscription and return the results in the requested template."""

    return _update(request, False, template, result_template, form=form)

def update_subscription(request, template="newsletter/update_form.html", 
    result_template="newsletter/update_results.html", form=NewsletterForm):
    """Add a subscription and return the results in the requested template."""

    return _update(request, 'FORM', template, result_template, form=form)

def _update(request, state, template, result_template, form=NewsletterForm):
    """Add a subscription and return the results in the requested template."""
    success = False
    result = ""

    if request.method == "POST":
        workform = form(request.POST)
        if workform.is_valid():
            if state == 'FORM':
                # save with subcription status from form
                result = workform.save()
            else:
                # save with subscription status explicitly set
                result = workform.save(state)
            success = True
        else:
            result = ugettext('Error, not valid.')

    else:
        workform = form()

    ctx = RequestContext(request, {
        'result' : result,
        'form' : workform
    })

    if success:
        return render_to_response(result_template, context_instance=ctx)
    else:
        return render_to_response(template, context_instance=ctx)



from django.contrib import admin
from product.modules.subscription.models import SubscriptionProduct, Trial

class Trial_Inline(admin.StackedInline):
    model = Trial
    extra = 2


class SubscriptionProductOptions(admin.ModelAdmin):
    inlines = [Trial_Inline]

admin.site.register(SubscriptionProduct, SubscriptionProductOptions)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#import os
#import traceback
import sys

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from notmm.controllers.wsgi             import BaseController, WSGIController
from notmm.utils.django_settings        import SettingsProxy
from notmm.utils.wsgilib                import HTTPRequest, HTTPResponse
from werkzeug.test import Client as TestClient

from sandbox import global_conf as app_conf

settings = SettingsProxy(autoload=True).get_settings()

ResponseClass = HTTPResponse
RequestClass = HTTPRequest

#import logging_conf, logging
#logger = logging.getLogger(__name__)

def make_app(callback, *args, **kwargs):
    return callback(*args, **kwargs)

def setup_test_handlers(callback, settings):
    #if settings.DEBUG:
    #    print('function %s registered to %s' % (func_name, module))
    callback.registerWSGIHandlers(settings.CUSTOM_ERROR_HANDLERS)
    return None

def inittestpackage():
    if 'settings' in sys.modules:
        del sys.modules['settings']


class DjangoTestCase(unittest.TestCase):
    def setUp(self):
        import django
        django.setup()

# default test controller

class BaseControllerTestCase(unittest.TestCase):
    wsgi_app = BaseController
    wsgi_client = TestClient
    
    def setUp(self):
        self.callback = self.wsgi_app() #self.wsgi_app()
        setup_test_handlers(self.callback, settings)
    
        self.client = self.wsgi_client(self.callback)  

    def tearDown(self):
        self.callback = self.client = None
    
class WSGIControllerTestCase(BaseControllerTestCase):
    #
    # Core tests for the WSGIController extension
    #
    wsgi_app = WSGIController

    def __init__(self, methodName='runTest'):
        super(WSGIControllerTestCase, self).__init__(methodName)

        self.settingsClass = type(settings)
        self.callback = None

    #def test_base(self):
    #    #self.assertEqual(self.callback.request_class != None, True)
    #    if self.callback is not None:
    #        #self.assertEqual(self.callback.get_request() != None, True)
    #        self.assertEqual(
    #            isinstance(self.callback.request, RequestClass),
    #            True,
    #            type(self.callback.request)
    #        )

    #def test_custom_error_handlers(self):
    #    self.assertEqual(isinstance(self.settingsClass, SettingsProxy), True, self.settingsClass)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .response import HTTPResponse
from .request  import HTTPRequest
from .exc      import (
    HTTPClientError, 
    HTTPException, 
	HTTPUnauthorized, 
    HTTPRedirectResponse,
    HTTPNotFound
    )



# Copyright (C) 2007-2009 Etienne Robillard <robillard.etienne@gmail.com>
# All rights reserved.
#
# Please see the "LICENSE" and "NOTICE" files included in the source 
# distribution for details on licensing info.
"""Schema for the BlogEngine app version 7.

TODO:
* BlogEntry -- Can be published/hidden.
* BlogEntryImage -- A image that belongs to a attached BlogEntry.
* BlogEntry

queries:
* LatestBlogsByDate -- A Feed subclass
* LatestBlogsByAuthor -- etc...
* LatestBlogsByCategory 

General stuff:
* Author -- The author of the BlogEntry.

* Comment -- A comment. 

* ContentType -- Basic content types. (optional)
* Tag -- A tag.

* TaggedItem -- TODO! 
* Status -- A status.
* Slug -- A slug. 
* Category -- A category. (optional)
* Feed -- A RSSFeed20 (TODO)

"""

from schevo.schema import *
schevo.schema.prep(locals())

from notmm.utils.decorators import permalink
from markdown2 import markdown, MarkdownError

class SchevoIcon(E.Entity):

    _hidden = True

    name = f.string()
    data = f.image()

    _key(name)

class Category(E.Entity):
    name = f.string()
    description = f.string(required=False)
    slug = f.string(required=False)

    _key(name)
    _plural = "Categories"

    def x_as_slug(self):
        
        if self.slug:
            slug = self.slug
        else:
            slug = self.name.replace(' ', '-').lower()

        return unicode(slug)

class Comment(E.Entity):
    author = f.entity('Author')
    #TODO support more flexible content/object types...
    blogentry = f.entity('BlogEntry')
    message = f.string(multiline=True)
    published = f.boolean(required=False, default=False)
    pub_date = f.datetime()

    _key(author, blogentry)

class Author(E.Entity):
    name = f.string()
    # TODO: add a emailstring() method to support e-mail bytestrings in utf8
    email = f.string(required=False)
    homepage = f.string(required=False)
    twitter_username = f.string(required=False)

    _key(name)

    #def x_blogentries(self):
    #    return [casting.movie
    #            for casting in self.m.movie_castings()]

class Tag(E.Entity):

    name = f.string()
    blogentry = f.entity('BlogEntry')
    #description = f.string(required=False)

    _key(name, blogentry)

class BlogEntry(E.Entity):

    title = f.string()
    pub_date = f.datetime()
    author = f.entity('Author')
    category = f.entity('Category', required=False)
    short_description = f.string(multiline=False, required=False, default='')
    
    # TODO: add markdown or textile support here
    body = f.string(multiline=True)
    # /path/to/filename.rst 
    source = f.path(required=False)
    
    #Need a f.multiplechoices field type!
    #source_type = f.string(
    #    preferred_values=('rest', 'html', 'text'),
    #    required=False)
    
    published = f.boolean(required=False, default=False)
    slug = f.string(required=False)

    _key(title)
    _plural = "Blog Entries"

    def x_comments(self):
        return [(comment.author, 
        comment.message) for comment in self.m.comments()]

    def x_tags(self):
        return [tag for tag in self.m.tags()]

    def __unicode__(self):
        return u'%s: %s' % (self.author, self.title)

    #### public methods

    @permalink
    def get_absolute_url(self):
        # returns the canonical URL representation for this post
        
        category_slug = self.category.x.as_slug()
        return ('blogengine.views.details', (), dict(
            category=category_slug, 
            slug=self.slug))
    
    def convert_to_html(self, name='body'):
        """ convert to html with markdown """
        # XXX: support other wiki-like converters
        try:
            s = getattr(self, name)
            # Can only convert strings, not _UNASSIGNED or None fields.. 
            if isinstance(s, basestring):
                html = markdown(s)
            else:
                html = ''
        except (MarkdownError, AttributeError), e:
            # Error converting the document to html
            raise e

        return html

#E.Comment._sample = [
#    (('Etienne Robillard',), ('Pepe is not a pony!',), 
#      'this is a test comment!', True, '2009-05-17 12:54'),
#]
#E.Tag._sample = [
#    ('notmm', ('Pepe is not a pony!',)),
#] 
#E.BlogEntry._sample = [
#    ('This is a test using notmm-schevo', '2009-05-04', 
#    ('Etienne Robillard', ), 'Just a test', 'Long description here'),
#    ('2nd post!', '2009-05-04',
#    ('Etienne Robillard', ), '2nd test with notmm-schevo', 'so far so good :)')]




'''
This module provides support for defining several options on your Elixir
entities.  There are three different kinds of options that can be set
up, and for this there are three different statements: using_options_,
using_table_options_ and using_mapper_options_.

Alternatively, these options can be set on all Elixir entities by modifying
the `options_defaults` dictionary before defining any entity.

`using_options`
---------------
The 'using_options' DSL statement allows you to set up some additional
behaviors on your model objects, including table names, ordering, and
more.  To specify an option, simply supply the option as a keyword
argument onto the statement, as follows:

.. sourcecode:: python

    class Person(Entity):
        name = Field(Unicode(64))

        using_options(shortnames=True, order_by='name')

The list of supported arguments are as follows:

+---------------------+-------------------------------------------------------+
| Option Name         | Description                                           |
+=====================+=======================================================+
| ``inheritance``     | Specify the type of inheritance this entity must use. |
|                     | It can be one of ``single``, ``concrete`` or          |
|                     | ``multi``. Defaults to ``single``.                    |
|                     | Note that polymorphic concrete inheritance is         |
|                     | currently not implemented. See:                       |
|                     | http://www.sqlalchemy.org/docs/05/mappers.html        |
|                     | #mapping-class-inheritance-hierarchies for an         |
|                     | explanation of the different kinds of inheritances.   |
+---------------------+-------------------------------------------------------+
| ``abstract``        | Set 'abstract'=True to declare abstract entity.       |
|                     | Abstract base classes are useful when you want to put |
|                     | some common information into a number of other        |
|                     | entities. Abstract entity will not be used to create  |
|                     | any database table. Instead, when it is used as a base|
|                     | class for other entity, its fields will be added to   |
|                     | those of the child class.                             |
+---------------------+-------------------------------------------------------+
| ``polymorphic``     | Whether the inheritance should be polymorphic or not. |
|                     | Defaults to ``True``. The column used to store the    |
|                     | type of each row is named "row_type" by default. You  |
|                     | can change this by passing the desired name for the   |
|                     | column to this argument.                              |
+---------------------+-------------------------------------------------------+
| ``identity``        | Specify a custom polymorphic identity. When using     |
|                     | polymorphic inheritance, this value (usually a        |
|                     | string) will represent this particular entity (class) |
|                     | . It will be used to differentiate it from other      |
|                     | entities (classes) in your inheritance hierarchy when |
|                     | loading from the database instances of different      |
|                     | entities in that hierarchy at the same time.          |
|                     | This value will be stored by default in the           |
|                     | "row_type" column of the entity's table (see above).  |
|                     | You can either provide a                              |
|                     | plain string or a callable. The callable will be      |
|                     | given the entity (ie class) as argument and must      |
|                     | return a value (usually a string) representing the    |
|                     | polymorphic identity of that entity.                  |
|                     | By default, this value is automatically generated: it |
|                     | is the name of the entity lower-cased.                |
+---------------------+-------------------------------------------------------+
| ``metadata``        | Specify a custom MetaData for this entity.            |
|                     | By default, entities uses the global                  |
|                     | ``elixir.metadata``.                                  |
|                     | This option can also be set for all entities of a     |
|                     | module by setting the ``__metadata__`` attribute of   |
|                     | that module.                                          |
+---------------------+-------------------------------------------------------+
| ``autoload``        | Automatically load column definitions from the        |
|                     | existing database table.                              |
+---------------------+-------------------------------------------------------+
| ``tablename``       | Specify a custom tablename. You can either provide a  |
|                     | plain string or a callable. The callable will be      |
|                     | given the entity (ie class) as argument and must      |
|                     | return a string representing the name of the table    |
|                     | for that entity. By default, the tablename is         |
|                     | automatically generated: it is a concatenation of the |
|                     | full module-path to the entity and the entity (class) |
|                     | name itself. The result is lower-cased and separated  |
|                     | by underscores ("_"), eg.: for an entity named        |
|                     | "MyEntity" in the module "project1.model", the        |
|                     | generated table name will be                          |
|                     | "project1_model_myentity".                            |
+---------------------+-------------------------------------------------------+
| ``shortnames``      | Specify whether or not the automatically generated    |
|                     | table names include the full module-path              |
|                     | to the entity. If ``shortnames`` is ``True``, only    |
|                     | the entity name is used. Defaults to ``False``.       |
+---------------------+-------------------------------------------------------+
| ``auto_primarykey`` | If given as string, it will represent the             |
|                     | auto-primary-key's column name.  If this option       |
|                     | is True, it will allow auto-creation of a primary     |
|                     | key if there's no primary key defined for the         |
|                     | corresponding entity.  If this option is False,       |
|                     | it will disallow auto-creation of a primary key.      |
|                     | Defaults to ``True``.                                 |
+---------------------+-------------------------------------------------------+
| ``version_id_col``  | If this option is True, it will create a version      |
|                     | column automatically using the default name. If given |
|                     | as string, it will create the column using that name. |
|                     | This can be used to prevent concurrent modifications  |
|                     | to the entity's table rows (i.e. it will raise an     |
|                     | exception if it happens). Defaults to ``False``.      |
+---------------------+-------------------------------------------------------+
| ``order_by``        | How to order select results. Either a string or a     |
|                     | list of strings, composed of the field name,          |
|                     | optionally lead by a minus (for descending order).    |
+---------------------+-------------------------------------------------------+
| ``session``         | Specify a custom contextual session for this entity.  |
|                     | By default, entities uses the global                  |
|                     | ``elixir.session``.                                   |
|                     | This option takes a ``ScopedSession`` object or       |
|                     | ``None``. In the later case your entity will be       |
|                     | mapped using a non-contextual mapper which requires   |
|                     | manual session management, as seen in pure SQLAlchemy.|
|                     | This option can also be set for all entities of a     |
|                     | module by setting the ``__session__`` attribute of    |
|                     | that module.                                          |
+---------------------+-------------------------------------------------------+
| ``autosetup``       | DEPRECATED. Specify whether that entity will contain  |
|                     | automatic setup triggers.                             |
|                     | That is if this entity will be                        |
|                     | automatically setup (along with all other entities    |
|                     | which were already declared) if any of the following  |
|                     | condition happen: some of its attributes are accessed |
|                     | ('c', 'table', 'mapper' or 'query'), instanciated     |
|                     | (called) or the create_all method of this entity's    |
|                     | metadata is called. Defaults to ``False``.            |
+---------------------+-------------------------------------------------------+
| ``allowcoloverride``| Specify whether it is allowed to override columns.    |
|                     | By default, Elixir forbids you to add a column to an  |
|                     | entity's table which already exist in that table. If  |
|                     | you set this option to ``True`` it will skip that     |
|                     | check. Use with care as it is easy to shoot oneself   |
|                     | in the foot when overriding columns.                  |
+---------------------+-------------------------------------------------------+

For examples, please refer to the examples and unit tests.

`using_table_options`
---------------------
The 'using_table_options' DSL statement allows you to set up some
additional options on your entity table. It is meant only to handle the
options which are not supported directly by the 'using_options' statement.
By opposition to the 'using_options' statement, these options are passed
directly to the underlying SQLAlchemy Table object (both non-keyword arguments
and keyword arguments) without any processing.

For further information, please refer to the `SQLAlchemy table's documentation
<http://www.sqlalchemy.org/docs/05/reference/sqlalchemy/schema.html
#sqlalchemy.schema.Table>`_.

You might also be interested in the section about `constraints
<http://www.sqlalchemy.org/docs/05/metadata.html
#defining-constraints-and-indexes>`_.

`using_mapper_options`
----------------------
The 'using_mapper_options' DSL statement allows you to set up some
additional options on your entity mapper. It is meant only to handle the
options which are not supported directly by the 'using_options' statement.
By opposition to the 'using_options' statement, these options are passed
directly to the underlying SQLAlchemy mapper (as keyword arguments)
without any processing.

For further information, please refer to the `SQLAlchemy mapper
function's documentation
<http://www.sqlalchemy.org/docs/05/reference/orm/mapping.html
#sqlalchemy.orm.mapper>`_.

`using_options_defaults`
------------------------
The 'using_options_defaults' DSL statement allows you to set up some
default options on a custom base class. These will be used as the default value
for options of all its subclasses. Note that any option not set within the
using_options_defaults (nor specifically on a particular Entity) will use the
global defaults, so you don't have to provide a default value for all options,
but only those you want to change. Please also note that this statement does
not work on normal entities, and the normal using_options statement does not
work on base classes (because normal options do not and should not propagate to
the children classes).
'''

from sqlalchemy import Integer, String

from elixir.statements import ClassMutator

__doc_all__ = ['options_defaults']

OLD_M2MCOL_NAMEFORMAT = "%(tablename)s_%(key)s%(numifself)s"
ALTERNATE_M2MCOL_NAMEFORMAT = "%(inversename)s_%(key)s"

def default_m2m_column_formatter(data):
    if data['selfref']:
        return ALTERNATE_M2MCOL_NAMEFORMAT % data
    else:
        return OLD_M2MCOL_NAMEFORMAT % data

NEW_M2MCOL_NAMEFORMAT = default_m2m_column_formatter

# format constants
FKCOL_NAMEFORMAT = "%(relname)s_%(key)s"
M2MCOL_NAMEFORMAT = NEW_M2MCOL_NAMEFORMAT
CONSTRAINT_NAMEFORMAT = "%(tablename)s_%(colnames)s_fk"
MULTIINHERITANCECOL_NAMEFORMAT = "%(entity)s_%(key)s"

# other global constants
DEFAULT_AUTO_PRIMARYKEY_NAME = "id"
DEFAULT_AUTO_PRIMARYKEY_TYPE = Integer
DEFAULT_VERSION_ID_COL_NAME = "row_version"
DEFAULT_POLYMORPHIC_COL_NAME = "row_type"
POLYMORPHIC_COL_SIZE = 40
POLYMORPHIC_COL_TYPE = String(POLYMORPHIC_COL_SIZE)

# debugging/migration help
MIGRATION_TO_07_AID = False

#
options_defaults = dict(
    abstract=False,
    autosetup=False,
    inheritance='single',
    polymorphic=True,
    identity=None,
    autoload=False,
    tablename=None,
    shortnames=False,
    auto_primarykey=True,
    version_id_col=False,
    allowcoloverride=False,
    order_by=None,
    resolve_root=None,
    mapper_options={},
    table_options={}
)

valid_options = options_defaults.keys() + [
    'metadata',
    'session',
    'collection'
]


def using_options_defaults_handler(entity, **kwargs):
    for kwarg in kwargs:
        if kwarg not in valid_options:
            raise Exception("'%s' is not a valid option for Elixir entities."
                            % kwarg)

    # We use __dict__ instead of hasattr to not check its presence within the
    # parent, and thus update the parent dict instead of creating a local dict.
    if not entity.__dict__.get('options_defaults'):
        entity.options_defaults = {}
    entity.options_defaults.update(kwargs)


def using_options_handler(entity, *args, **kwargs):
    for kwarg in kwargs:
        if kwarg in valid_options:
            setattr(entity._descriptor, kwarg, kwargs[kwarg])
        else:
            raise Exception("'%s' is not a valid option for Elixir entities."
                            % kwarg)


def using_table_options_handler(entity, *args, **kwargs):
    entity._descriptor.table_args.extend(list(args))
    entity._descriptor.table_options.update(kwargs)


def using_mapper_options_handler(entity, *args, **kwargs):
    entity._descriptor.mapper_options.update(kwargs)


using_options_defaults = ClassMutator(using_options_defaults_handler)
using_options = ClassMutator(using_options_handler)
using_table_options = ClassMutator(using_table_options_handler)
using_mapper_options = ClassMutator(using_mapper_options_handler)



def delete_named_urlpattern(urlpatterns, name):
    """Delete the named urlpattern.
    """

    found = False
    ix = 0

    while not found and ix < len(urlpatterns):
        pattern = urlpatterns[ix]
        if hasattr(pattern, 'url_patterns'):
            found = delete_named_urlpattern(pattern.url_patterns, name)
        else:
            if name and hasattr(pattern, 'name'):
                if pattern.name == name:
                    del urlpatterns[ix]
                    found = True

        if not found:
            ix += 1

    return found

def remove_duplicate_urls(urls, names):
    """Remove any URLs whose names are already in use."""
    for pattern in urls:
        if hasattr(pattern, 'url_patterns'):
            remove_duplicate_urls(pattern.url_patterns, names)

        elif hasattr(pattern, 'name') and pattern.name:
            if pattern.name in names:
                urls.remove(pattern)
            else:
                names.append(pattern.name)

def replace_urlpattern(urlpatterns, replacement):
    """Delete the old urlpattern, and add a new one.
    
    parameters:
        urlpatterns: list
        replacement: an `django.conf.urls.defaults.url` object.
    
    example:
    
        replacement = url(r'^accounts/login/', 'my.site.login_signup', {}, name='auth_login'))
        replace_urlpattern(urlpatterns, replacement)
    """

    found = False
    ix = 0
    if hasattr(replacement, 'name') and replacement.name:
        name = replacement.name
        regex = None
    else:
        name = None
        regex = replacement.regex.pattern
    
    while not found and ix < len(urlpatterns):
        pattern = urlpatterns[ix]
        if hasattr(pattern, 'url_patterns'):
            found = replace_urlpattern(pattern.url_patterns, replacement)
        else:
            if name and hasattr(pattern, 'name'):
                if pattern.name == name:
                    del urlpatterns[ix]
                    urlpatterns.append(replacement)
                    found = True
            elif pattern.regex.pattern == regex:
                del urlpatterns[ix]
                urlpatterns.append(replacement)
                found = True
                    
        if not found:
            ix += 1
            
    return found
    
def replace_urlpatterns(urlpatterns, replacelist):
    for replace in replacelist:
        replace_urlpattern(urlpatterns, replace)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2019 Etienne Robillard <tkadm30@yandex.ru>
# See NOTICE file for details. 
"""BaseController API Version 0.9.8 revision 0

Python extension module to implement MVC-style "controllers"
for Django and WSGI type apps. In short, ``BaseController`` derived
extensions are request handlers resolving a ``path_url`` string
to a matching ``view function``. The response handler (view function)
then resolve the appropriate HTTP response to the client.

TODO:
-Better module-level documentation (work-in-progress)
-Compatibility with mod_wsgi (Apache2) (YMMV...)
-Change all prints by log() hooks (work-in-progress)
-use signals to log messages from pubsub like BUS

Define and document the internal request handling stages when
using native Django views (WSGIHandler):
 __init__,         # application init -> self.init_request(env)
 init_request,     # setup the environment -> self.process_request(req)
 process_request,  # handle the request -> self.locals.request = req
 get_response,     # resolve request [PATH_INFO] -> response_callback
 application,      # response stage 2 [WSGI] -> response_callback(env, start_response)
"""

import sys
import os
import urllib
import logging
import traceback

from contextlib import contextmanager
from notmm.utils.wsgilib import (
    HTTPRequest, 
    HTTPResponse,
    #HTTPNotFound,
    #HTTPUnauthorized,
    #HTTPException
    )
from notmm.utils.wsgilib.exc import (
    HTTPNotFound, 
    HTTPUnauthorized, 
    HTTPException
    )

from notmm.utils.django_compat import NoReverseMatch

from werkzeug.local import Local, LocalProxy

RequestClass = HTTPRequest

_local = Local()
#local_manager = LocalManager([_local]) 

__all__ = ('BaseController', 'sessionmanager', 'get_current_request')


@contextmanager
def sessionmanager(request):
    _local.request = request
    yield
    _local.request = None

def get_current_request():
    try:
        return _local.request
    except AttributeError:
        raise TypeError("No request object for this thread")

_request = LocalProxy(lambda: get_current_request())

class BaseController(object):
    debug = False
    request_class = RequestClass
    response_class = HTTPResponse
    _response = None

    def sethandle(self, name, string_or_callable):
        """Adds custom response handlers to a BaseController subclass.
        """

        # If string_or_callable is in the form of "foo.bar.quux",
        # the callable function should be the last part of the
        # string
        handler = None
        if isinstance(string_or_callable, str):
            if string_or_callable.find('.') != -1:
                bits = string_or_callable.rsplit('.', 1)
                m = __import__(bits[0], globals(), locals(), fromlist=[''])
                for component in bits[1:]:
                    if hasattr(m, component):
                        handler = getattr(m, component)
                        break
                    else:
                        #print 'debug: module %s has no such member: %r' % (m, component)
                        continue
        elif callable(string_or_callable):
            handler = string_or_callable
        else:
            raise ValueError("Unexpected string_or_callable type: %r" % \
                type(string_or_callable))

        if not callable(handler) and isinstance(handler, str):
            # Attempt to import it
            # XXX write a native eval_import hook
            if 'debug.config.eval_import_func' in self.app_conf:
                handler_obj = self.app_conf['debug.config.eval_import_func'](handler)
            else:
                raise Exception("fatal error: debug.config.eval_import_func not set!")
        else:
            handler_obj = handler

        # XXX Use staticmethod(func) here, because we want to support the
        # same positional arguments named by this callable
        setattr(self, name, handler_obj)

        return None

    def __call__(self, environ, start_response, exc_info=None):
        # WSGI 1.0 mandates to return a callable object
        #self._environ.update(environ)
        if exc_info is not None:
            # TODO: exc_info verifications
            if self.debug:
                self.logger.debug(exc_info)
            return self.application(environ, start_response, exc_info)
        return self.application(environ, start_response)

    def application(self, environ, start_response):
        """
        Override this to change the default request/response
        handling.

        This method is used for properly returning a WSGI response instance
        by calling ``get_response``.

        The latter does the grunt work of routing the request to the
        proper callable function or class.

        """
        req = self.request_class(environ)

        with sessionmanager(req):
            
            self._request = self.init_request(req)
        
            if self.debug:
                assert _request == self.request, 'Invalid request object!'
                assert _request == get_current_request(), 'Request is not the current one!'
            try:
                self._response = self.get_response(request=self.request)
            except HTTPException:
                #if self.debug:
                #    assert isinstance(self._response, self.response_class), 'wrong response type!'
                return self.handle500(request)
        return self._response(environ, start_response)

    def registerWSGIHandlers(self, d):
        """Register appropriate wsgi callbacks (legacy method
        for backward compat only)"""
        for k, v in d:
            # "register" the callback function as a standard Django view
            # accessible by the controller extension
            self.sethandle(k, v)
        #self.registered = True
        return None
    
    def init_request(self, request):
        """A method to execute before ``process_request``"""
        # put handle404 and handle500 in request.environ
        if hasattr(self, 'handle404'):
            request.environ['django.request.handle404'] = self.handle404
        if hasattr(self, 'handle500'):
            request.environ['django.request.handle500'] = self.handle500
        return request    

    @property
    def request(self):
        return self._request


    def get_response(self, request, method='GET', data={}):
        """Process ``path_url`` and return a callable function as
        the WSGI ``response`` callback.

        The callback view function is resolved using the built-in
        Django ``RegexURLResolver`` class.

        Returns a callable function (Response) or None if no
        view functions matched.

        See the docs in :notmm.utils.django_compat.RegexURLResolver:
        for details.

        This function may be overrided in custom subclasses to modify
        the response type.
        """
        # Match the location to a view or callable
        #path_info = self.get_path_info(self.environ)
        
        if self.debug:
            assert hasattr(request, 'path_url'), 'something is broken!'
            #assert isinstance(request, self.request_class), 'Unknown request class: %s' % type(request)

        try:
            if self.debug:
                #assert self.request != None, 'invalid request object!'
                self.logger.debug("Resolving path=%r"%request.path_url)

            # If the path doesn't end with a slash, redirect to the same path
            # with an ending slash.
            if (not request.path_url.endswith('/') 
                and self.settings.APPEND_SLASH == True):
                redirect_url = request.path_url + '/'
                return self.handle302(request, location=redirect_url)

            # Resolve the path (endpoint) to a view using legacy Django URL
            # resolver.
            (callback, args, kwargs) = self.resolver.resolve(str(request.path_url))
            if self.debug and self.logger:
                self.logger.debug("callback resolved=%r"%callback)
            # Create the wsgi ``response`` object.
            # If `callback` here is a decorator type we're fuck and it will returns a
            # http request object.
            response = callback(request, *args, **kwargs)
            #self.logger.debug("new response type: %s" % type(response))
        except NoReverseMatch as e:
            # Handle 404 responses with a custom 404 handler.
            self.logger.info('Document not found=%s'%request.path_url)
            if self.debug:
                self.logger.debug(e)
            #handleclienterror(self.request)

            return self.handle404(request)
        except HTTPException:
            if self.debug:
                exc = traceback.format_exc()
                self.logger.debug(exc)
            return self.handle500(request)
        except HTTPUnauthorized as exc:
            if self.debug:
                self.logger.debug("Caught authorization exception!")
                self.logger.debug(exc)
            return self.handle401(request)
        else:
            return response
    
    @property
    def response(self):
        return self._response

    @property
    def user(self):
        return self.request.get_user()

    def _environ_getter(self):
        """ Returns the current WSGI environment instance."""
        return getattr(self, '_environ', {})

    environ = property(_environ_getter)

    def _method_getter(self):
        return self.environ['REQUEST_METHOD']
    method = property(_method_getter)

    def _debug_getter(self):
        """Global debug flag. 
        Set settings.DEBUG to False to disable debugging"""
        return bool(self.settings.DEBUG == True)
    debug = property(_debug_getter)

    def _get_path_info(self, env):
        return str(env.get('PATH_INFO', ''))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved.
#,
'''Light e-mail utilities to send mail with Django.

'''

from django.core.mail import send_mail, EmailMultiAlternatives
from smtplib import SMTPConnectError
import socket

__all__ = ['send_mail', 'send_html_mail']

def send_mail(recipients, from_addr=None, subject=None, message=None):  
    """Sends a e-mail message to a list of recipients.
    
    Returns ``None`` to indicate success sending the message.
    """
    
    try:
        send_mail(subject, message, from_addr, recipients)
    except (SMTPConnectError, socket.error), e:
        # temporary problem connecting to the smtp server
        raise e
    
    return None

def send_html_mail(recipients, from_addr, 
    subject=None, message=None, content_type='text/html',
    html_message_class=EmailMultiAlternatives
    ):
    """Sends a HTML message using the ``EmailMultiAlternatives`` class."""
    email = html_message_class(subject, message, from_addr, recipients)
    email.attach_alternative(message, content_type)
    email.send()
    
    return None

